# Stubs for galaxy.jobs.metrics.instrumenters.uname (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import formatting
from ..instrumenters import InstrumentPlugin

class UnameFormatter(formatting.JobMetricFormatter):
    def format(self, key, value): ...

class UnamePlugin(InstrumentPlugin):
    plugin_type = ...  # type: str
    formatter = ...  # type: Any
    uname_args = ...  # type: Any
    def __init__(self, **kwargs) -> None: ...
    def pre_execute_instrument(self, job_directory): ...
    def job_properties(self, job_id, job_directory): ...



# Stubs for galaxy.tools.deps.docker_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from .commands import argv_to_str as argv_to_str

DEFAULT_DOCKER_COMMAND = ...  # type: str
DEFAULT_SUDO = ...  # type: bool
DEFAULT_SUDO_COMMAND = ...  # type: str
DEFAULT_HOST = ...  # type: Any
DEFAULT_VOLUME_MOUNT_TYPE = ...  # type: str
DEFAULT_WORKING_DIRECTORY = ...  # type: Any
DEFAULT_NET = ...  # type: Any
DEFAULT_MEMORY = ...  # type: Any
DEFAULT_VOLUMES_FROM = ...  # type: Any
DEFAULT_AUTO_REMOVE = ...  # type: bool
DEFAULT_SET_USER = ...  # type: str
DEFAULT_RUN_EXTRA_ARGUMENTS = ...  # type: Any

class DockerVolume:
    from_path = ...  # type: Any
    to_path = ...  # type: Any
    how = ...  # type: Any
    def __init__(self, path, to_path: Optional[Any] = ..., how: Any = ...) -> None: ...
    @staticmethod
    def volumes_from_str(volumes_as_str): ...
    @staticmethod
    def volume_from_str(as_str): ...

def kill_command(container, signal: Optional[Any] = ..., **kwds): ...
def logs_command(container, **kwds): ...
def build_command(image, docker_build_path, **kwds): ...
def build_save_image_command(image, destination, **kwds): ...
def build_pull_command(tag, **kwds): ...
def build_docker_cache_command(image, **kwds): ...
def build_docker_images_command(truncate: bool = ..., **kwds): ...
def build_docker_load_command(**kwds): ...
def build_docker_run_command(container_command, image, interactive: bool = ..., terminal: bool = ..., tag: Optional[Any] = ..., volumes: Any = ..., volumes_from: Any = ..., memory: Any = ..., env_directives: Any = ..., working_directory: Any = ..., name: Optional[Any] = ..., net: Any = ..., run_extra_arguments: Any = ..., docker_cmd: Any = ..., sudo: Any = ..., sudo_cmd: Any = ..., auto_rm: Any = ..., set_user: Any = ..., host: Any = ...): ...
def command_list(command, command_args: Any = ..., **kwds): ...
def command_shell(command, command_args: Any = ..., **kwds): ...



# Stubs for networkx.algorithms.vitality (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def closeness_vitality(G, node: Optional[Any] = ..., weight: Optional[Any] = ..., wiener_index: Optional[Any] = ...): ...



# Stubs for networkx.algorithms.flow.preflowpush (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def preflow_push(G, s, t, capacity: str = ..., residual: Optional[Any] = ..., global_relabel_freq: int = ..., value_only: bool = ...): ...



# Stubs for networkx.linalg.graphmatrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def incidence_matrix(G, nodelist: Optional[Any] = ..., edgelist: Optional[Any] = ..., oriented: bool = ..., weight: Optional[Any] = ...): ...
def adjacency_matrix(G, nodelist: Optional[Any] = ..., weight: str = ...): ...
adj_matrix = adjacency_matrix



# Stubs for networkx.algorithms.connectivity.disjoint_paths (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.flow import edmonds_karp
from typing import Any, Optional

default_flow_func = edmonds_karp

def edge_disjoint_paths(G, s, t, flow_func: Optional[Any] = ..., cutoff: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ...): ...
def node_disjoint_paths(G, s, t, flow_func: Optional[Any] = ..., cutoff: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ...): ...



# Stubs for galaxy.tools.deps.mulled.util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from collections import namedtuple

def quay_versions(namespace, pkg_name): ...
def mulled_tags_for(namespace, image, tag_prefix: Optional[Any] = ...): ...
def split_tag(tag): ...
def version_sorted(elements): ...

Target = namedtuple('Target', ['package_name', 'version', 'build'])

def build_target(package_name, version: Optional[Any] = ..., build: Optional[Any] = ..., tag: Optional[Any] = ...): ...
def conda_build_target_str(target): ...
def v1_image_name(targets, image_build: Optional[Any] = ..., name_override: Optional[Any] = ...): ...
def v2_image_name(targets, image_build: Optional[Any] = ..., name_override: Optional[Any] = ...): ...

image_name = ...  # type: Any

# Names in __all__ with no definition:
#   Target



# Stubs for galaxy.tools.deps.conda_compat (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from conda_build.metadata import MetaData as MetaData

MetaData = ...  # type: Any

class _Memoized:
    func = ...  # type: Any
    cache = ...  # type: Any
    def __init__(self, func) -> None: ...
    def __call__(self, *args): ...

def raw_metadata(recipe_dir): ...

class _MetaData:
    meta = ...  # type: Any
    def __init__(self, input_dir) -> None: ...
    def get_value(self, field, default: Optional[Any] = ...): ...



# Stubs for galaxy.tools.cwl.parser (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def tool_proxy(tool_path, strict_cwl_validation: bool = ...): ...
def load_job_proxy(job_directory, strict_cwl_validation: bool = ...): ...

class ToolProxy:
    def __init__(self, tool, tool_path) -> None: ...
    def job_proxy(self, input_dict, output_dict, job_directory: str = ...): ...
    def input_instances(self): ...
    def output_instances(self): ...
    def docker_identifier(self): ...
    def description(self): ...
    def label(self): ...

class CommandLineToolProxy(ToolProxy):
    def description(self): ...
    def label(self): ...
    def input_instances(self): ...
    def output_instances(self): ...
    def docker_identifier(self): ...

class ExpressionToolProxy(CommandLineToolProxy): ...

class JobProxy:
    def __init__(self, tool_proxy, input_dict, output_dict, job_directory) -> None: ...
    def cwl_job(self): ...
    @property
    def is_command_line_job(self): ...
    @property
    def command_line(self): ...
    @property
    def stdin(self): ...
    @property
    def stdout(self): ...
    @property
    def environment(self): ...
    @property
    def generate_files(self): ...
    def collect_outputs(self, tool_working_directory): ...
    def save_job(self): ...
    def output_id(self, output_name): ...
    def output_path(self, output_name): ...
    def output_secondary_files_dir(self, output_name, create: bool = ...): ...
    def stage_files(self): ...

class WorkflowProxy:
    def __init__(self, workflow, workflow_path) -> None: ...
    def step_proxies(self): ...
    @property
    def runnables(self): ...
    def to_dict(self): ...

class StepProxy:
    def __init__(self, workflow_proxy, step) -> None: ...
    def to_dict(self): ...

class ConditionalInstance:
    input_type = ...  # type: Any
    name = ...  # type: Any
    case = ...  # type: Any
    whens = ...  # type: Any
    def __init__(self, name, case, whens) -> None: ...
    def to_dict(self): ...

class SelectInputInstance:
    input_type = ...  # type: Any
    name = ...  # type: Any
    label = ...  # type: Any
    description = ...  # type: Any
    options = ...  # type: Any
    def __init__(self, name, label, description, options) -> None: ...
    def to_dict(self): ...

class InputInstance:
    input_type = ...  # type: Any
    name = ...  # type: Any
    label = ...  # type: Any
    description = ...  # type: Any
    required = ...  # type: bool
    array = ...  # type: Any
    area = ...  # type: Any
    def __init__(self, name, label, description, input_type, array: bool = ..., area: bool = ...) -> None: ...
    def to_dict(self, itemwise: bool = ...): ...

class OutputInstance:
    name = ...  # type: Any
    output_data_type = ...  # type: Any
    output_type = ...  # type: Any
    path = ...  # type: Any
    def __init__(self, name, output_data_type, output_type, path: Optional[Any] = ...) -> None: ...



# Stubs for galaxy.tools.deps.dependencies (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class AppInfo:
    galaxy_root_dir = ...  # type: Any
    default_file_path = ...  # type: Any
    outputs_to_working_directory = ...  # type: Any
    container_image_cache_path = ...  # type: Any
    library_import_dir = ...  # type: Any
    enable_mulled_containers = ...  # type: Any
    containers_resolvers_config_file = ...  # type: Any
    involucro_path = ...  # type: Any
    involucro_auto_init = ...  # type: Any
    def __init__(self, galaxy_root_dir: Optional[Any] = ..., default_file_path: Optional[Any] = ..., outputs_to_working_directory: bool = ..., container_image_cache_path: Optional[Any] = ..., library_import_dir: Optional[Any] = ..., enable_mulled_containers: bool = ..., containers_resolvers_config_file: Optional[Any] = ..., involucro_path: Optional[Any] = ..., involucro_auto_init: bool = ...) -> None: ...

class DependenciesDescription:
    requirements = ...  # type: Any
    installed_tool_dependencies = ...  # type: Any
    def __init__(self, requirements: Any = ..., installed_tool_dependencies: Any = ...) -> None: ...
    def to_dict(self): ...
    @staticmethod
    def from_dict(as_dict): ...

class ToolInfo:
    container_descriptions = ...  # type: Any
    requirements = ...  # type: Any
    requires_galaxy_python_environment = ...  # type: Any
    env_pass_through = ...  # type: Any
    def __init__(self, container_descriptions: Any = ..., requirements: Any = ..., requires_galaxy_python_environment: bool = ...) -> None: ...



# Stubs for prov.serializers (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov import Error
from typing import Any, Optional

class Serializer:
    document: Any = ...
    def __init__(self, document: Optional[Any] = ...) -> None: ...
    def serialize(self, stream: Any, **kwargs: Any) -> None: ...
    def deserialize(self, stream: Any, **kwargs: Any) -> None: ...

class DoNotExist(Error): ...

class Registry:
    serializers: Any = ...
    @staticmethod
    def load_serializers() -> None: ...

def get(format_name: Any): ...



# Stubs for galaxy.util.odict (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from six.moves import UserDict

dict_alias = ...  # type: Any

class odict(UserDict):
    def __init__(self, dict: Optional[Any] = ...) -> None: ...
    def __delitem__(self, key): ...
    def __setitem__(self, key, item): ...
    def clear(self): ...
    def copy(self): ...
    def items(self): ...
    def keys(self): ...
    def popitem(self): ...
    def setdefault(self, key, failobj: Optional[Any] = ...): ...
    def update(self, dict): ...
    def values(self): ...
    def iterkeys(self): ...
    def itervalues(self): ...
    def iteritems(self): ...
    def __iter__(self): ...
    def reverse(self): ...
    def insert(self, index, key, item): ...



# Stubs for galaxy.tools.verify.asserts.text (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def assert_has_text(output, text): ...
def assert_not_has_text(output, text): ...
def assert_has_line(output, line): ...
def assert_has_text_matching(output, expression): ...
def assert_has_line_matching(output, expression): ...



# Stubs for galaxy.tools.deps.conda_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import installable
from sys import platform as _platform

class CondaContext(installable.InstallableContext):
    installable_description = ...  # type: str
    condarc_override = ...  # type: Any
    conda_exec = ...  # type: Any
    debug = ...  # type: Any
    shell_exec = ...  # type: Any
    copy_dependencies = ...  # type: Any
    conda_prefix = ...  # type: Any
    ensure_channels = ...  # type: Any
    ensured_channels = ...  # type: bool
    use_local = ...  # type: Any
    def __init__(self, conda_prefix: Optional[Any] = ..., conda_exec: Optional[Any] = ..., shell_exec: Optional[Any] = ..., debug: bool = ..., ensure_channels: str = ..., condarc_override: Optional[Any] = ..., use_path_exec: Any = ..., copy_dependencies: bool = ..., use_local: Any = ...) -> None: ...
    @property
    def conda_version(self): ...
    @property
    def conda_build_available(self): ...
    def ensure_channels_configured(self): ...
    def ensure_conda_build_installed_if_needed(self): ...
    def conda_info(self): ...
    def is_conda_installed(self): ...
    def can_install_conda(self): ...
    def load_condarc(self): ...
    def save_condarc(self, conf): ...
    @property
    def condarc(self): ...
    def command(self, operation, args): ...
    def exec_command(self, operation, args): ...
    def exec_create(self, args, allow_local: bool = ...): ...
    def exec_remove(self, args): ...
    def exec_install(self, args, allow_local: bool = ...): ...
    def exec_clean(self, args: Any = ..., quiet: bool = ...): ...
    def export_list(self, name, path): ...
    def env_path(self, env_name): ...
    @property
    def envs_path(self): ...
    def has_env(self, env_name): ...
    @property
    def deactivate(self): ...
    @property
    def activate(self): ...
    def is_installed(self): ...
    def can_install(self): ...
    @property
    def parent_path(self): ...

class CondaTarget:
    package = ...  # type: Any
    version = ...  # type: Any
    channel = ...  # type: Any
    def __init__(self, package, version: Optional[Any] = ..., channel: Optional[Any] = ...) -> None: ...
    @property
    def package_specifier(self): ...
    @property
    def install_environment(self): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

def install_conda(conda_context: Optional[Any] = ..., force_conda_build: bool = ...): ...
def install_conda_target(conda_target, conda_context: Optional[Any] = ..., skip_environment: bool = ...): ...
def requirements_to_conda_targets(requirements, conda_context: Optional[Any] = ...): ...



# Stubs for galaxy.tools.parser.xml (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from .interface import InputSource as InputSource, PageSource as PageSource, PagesSource as PagesSource, TestCollectionDef as TestCollectionDef, TestCollectionOutputDef as TestCollectionOutputDef, ToolSource as ToolSource, ToolStdioExitCode as ToolStdioExitCode, ToolStdioRegex as ToolStdioRegex
from .output_actions import ToolOutputActionGroup as ToolOutputActionGroup
from .output_collection_def import dataset_collector_descriptions_from_elem as dataset_collector_descriptions_from_elem
from .output_objects import ToolOutput as ToolOutput, ToolOutputCollection as ToolOutputCollection, ToolOutputCollectionStructure as ToolOutputCollectionStructure
from .util import aggressive_error_checks as aggressive_error_checks, error_on_exit_code as error_on_exit_code

log = ...  # type: Any

class XmlToolSource(ToolSource):
    xml_tree = ...  # type: Any
    root = ...  # type: Any
    legacy_defaults = ...  # type: Any
    def __init__(self, xml_tree, source_path: Optional[Any] = ...) -> None: ...
    def parse_version(self): ...
    def parse_id(self): ...
    def parse_tool_module(self): ...
    def parse_action_module(self): ...
    def parse_tool_type(self): ...
    def parse_name(self): ...
    def parse_edam_operations(self): ...
    def parse_edam_topics(self): ...
    def parse_description(self): ...
    def parse_is_multi_byte(self): ...
    def parse_display_interface(self, default): ...
    def parse_require_login(self, default): ...
    def parse_request_param_translation_elem(self): ...
    def parse_command(self): ...
    def parse_environment_variables(self): ...
    def parse_interpreter(self): ...
    def parse_version_command(self): ...
    def parse_version_command_interpreter(self): ...
    def parse_parallelism(self): ...
    def parse_hidden(self): ...
    def parse_redirect_url_params_elem(self): ...
    def parse_sanitize(self): ...
    def parse_refresh(self): ...
    def parse_requirements_and_containers(self): ...
    def parse_input_pages(self): ...
    def parse_outputs(self, tool): ...
    def parse_stdio(self): ...
    def parse_strict_shell(self): ...
    def parse_help(self): ...
    def parse_tests_to_dict(self): ...
    def parse_profile(self): ...

class StdioParser:
    stdio_exit_codes = ...  # type: Any
    stdio_regexes = ...  # type: Any
    def __init__(self, root) -> None: ...
    def parse_stdio_exit_codes(self, stdio_elem): ...
    def parse_stdio_regexes(self, stdio_elem): ...
    def parse_error_level(self, err_level): ...

class XmlPagesSource(PagesSource):
    input_elem = ...  # type: Any
    def __init__(self, root) -> None: ...
    @property
    def inputs_defined(self): ...

class XmlPageSource(PageSource):
    parent_elem = ...  # type: Any
    def __init__(self, parent_elem) -> None: ...
    def parse_display(self): ...
    def parse_input_sources(self): ...

class XmlInputSource(InputSource):
    input_elem = ...  # type: Any
    input_type = ...  # type: Any
    def __init__(self, input_elem) -> None: ...
    def parse_input_type(self): ...
    def elem(self): ...
    def get(self, key, value: Optional[Any] = ...): ...
    def get_bool(self, key, default): ...
    def parse_label(self): ...
    def parse_help(self): ...
    def parse_sanitizer_elem(self): ...
    def parse_validator_elems(self): ...
    def parse_dynamic_options_elem(self): ...
    def parse_static_options(self): ...
    def parse_optional(self, default: Optional[Any] = ...): ...
    def parse_conversion_tuples(self): ...
    def parse_nested_inputs_source(self): ...
    def parse_test_input_source(self): ...
    def parse_when_input_sources(self): ...



# Stubs for galaxy (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

PROJECT_NAME = ...  # type: str
PROJECT_OWNER = ...  # type: str
PROJECT_USERAME = ...  # type: str
PROJECT_URL = ...  # type: str
PROJECT_AUTHOR = ...  # type: str
PROJECT_EMAIL = ...  # type: str
RAW_CONTENT_URL = ...  # type: Any



# Stubs for galaxy.util.oset (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import collections

class OrderedSet(collections.MutableSet):
    end = ...  # type: Any
    map = ...  # type: Any
    def __init__(self, iterable: Optional[Any] = ...) -> None: ...
    def __len__(self): ...
    def __contains__(self, key): ...
    def add(self, key): ...
    def discard(self, key): ...
    def __iter__(self): ...
    def __reversed__(self): ...
    def pop(self, last: bool = ...): ...
    def __eq__(self, other): ...



# Stubs for galaxy.tools.cwl.schema (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .cwltool_deps import ensure_cwltool_available as ensure_cwltool_available, load_tool as load_tool, schema_salad as schema_salad, workflow as workflow
from collections import namedtuple

RawProcessReference = namedtuple('RawProcessReference', ['process_object', 'uri'])

ProcessDefinition = namedtuple('ProcessDefinition', ['process_object', 'metadata', 'document_loader', 'avsc_names', 'raw_process_reference'])

class SchemaLoader:
    def __init__(self, strict: bool = ...) -> None: ...
    @property
    def raw_document_loader(self): ...
    def raw_process_reference(self, path): ...
    def process_definition(self, raw_reference): ...
    def tool(self, **kwds): ...

schema_loader = ...  # type: Any
non_strict_schema_loader = ...  # type: Any



# Stubs for galaxy.tools.deps.resolvers.modules (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..resolvers import Dependency, DependencyResolver, MappableDependencyResolver

class ModuleDependencyResolver(DependencyResolver, MappableDependencyResolver):
    dict_collection_visible_keys = ...  # type: Any
    resolver_type = ...  # type: str
    versionless = ...  # type: Any
    modulecmd = ...  # type: Any
    modulepath = ...  # type: Any
    default_indicator = ...  # type: Any
    module_checker = ...  # type: Any
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def resolve(self, requirement, **kwds): ...

class DirectoryModuleChecker:
    module_dependency_resolver = ...  # type: Any
    directories = ...  # type: Any
    def __init__(self, module_dependency_resolver, modulepath, prefetch) -> None: ...
    def has_module(self, module, version): ...

class AvailModuleChecker:
    module_dependency_resolver = ...  # type: Any
    modulepath = ...  # type: Any
    default_indicator = ...  # type: Any
    prefetched_modules = ...  # type: Any
    def __init__(self, module_dependency_resolver, modulepath, prefetch, default_indicator: Any = ...) -> None: ...
    def has_module(self, module, version): ...

class ModuleDependency(Dependency):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: str
    module_dependency_resolver = ...  # type: Any
    module_name = ...  # type: Any
    module_version = ...  # type: Any
    def __init__(self, module_dependency_resolver, module_name, module_version: Optional[Any] = ..., exact: bool = ...) -> None: ...
    @property
    def exact(self): ...
    def shell_commands(self, requirement): ...



# Stubs for galaxy.tools.deps.resolvers.unlinked_tool_shed_packages (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .galaxy_packages import BaseGalaxyPackageDependencyResolver
from ..resolvers import Dependency

class UnlinkedToolShedPackageDependencyResolver(BaseGalaxyPackageDependencyResolver):
    dict_collection_visible_keys = ...  # type: Any
    resolver_type = ...  # type: str
    preferred_owners = ...  # type: Any
    select_by_owner = ...  # type: Any
    def __init__(self, dependency_manager, **kwds) -> None: ...

class CandidateDependency(Dependency):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: str
    @property
    def exact(self): ...
    dependency = ...  # type: Any
    path = ...  # type: Any
    owner = ...  # type: Any
    def __init__(self, dependency, path, owner: Any = ...) -> None: ...
    def shell_commands(self, requirement): ...



# Stubs for networkx.linalg.attrmatrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def attr_matrix(G, edge_attr: Optional[Any] = ..., node_attr: Optional[Any] = ..., normalized: bool = ..., rc_order: Optional[Any] = ..., dtype: Optional[Any] = ..., order: Optional[Any] = ...): ...
def attr_sparse_matrix(G, edge_attr: Optional[Any] = ..., node_attr: Optional[Any] = ..., normalized: bool = ..., rc_order: Optional[Any] = ..., dtype: Optional[Any] = ...): ...



# Stubs for galaxy.tools.parser.output_collection_def (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

DEFAULT_EXTRA_FILENAME_PATTERN = ...  # type: str
DEFAULT_SORT_BY = ...  # type: str
DEFAULT_SORT_COMP = ...  # type: str
NAMED_PATTERNS = ...  # type: Any
INPUT_DBKEY_TOKEN = ...  # type: str
LEGACY_DEFAULT_DBKEY = ...  # type: Any

def dataset_collector_descriptions_from_elem(elem, legacy: bool = ...): ...
def dataset_collector_descriptions_from_list(discover_datasets_dicts): ...

class DatasetCollectionDescription:
    pattern = ...  # type: Any
    default_dbkey = ...  # type: Any
    default_ext = ...  # type: Any
    default_visible = ...  # type: Any
    directory = ...  # type: Any
    assign_primary_output = ...  # type: Any
    sort_reverse = ...  # type: bool
    sort_key = ...  # type: Any
    sort_comp = ...  # type: Any
    def __init__(self, **kwargs) -> None: ...

DEFAULT_DATASET_COLLECTOR_DESCRIPTION = ...  # type: Any



# Stubs for galaxy.tools.parser.output_objects (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from galaxy.util.dictifiable import Dictifiable

class ToolOutputBase(Dictifiable):
    name = ...  # type: Any
    label = ...  # type: Any
    filters = ...  # type: Any
    hidden = ...  # type: Any
    collection = ...  # type: bool
    def __init__(self, name, label: Optional[Any] = ..., filters: Optional[Any] = ..., hidden: bool = ...) -> None: ...

class ToolOutput(ToolOutputBase):
    dict_collection_visible_keys = ...  # type: Any
    format = ...  # type: Any
    format_source = ...  # type: Any
    metadata_source = ...  # type: Any
    parent = ...  # type: Any
    actions = ...  # type: Any
    change_format = ...  # type: Any
    implicit = ...  # type: Any
    from_work_dir = ...  # type: Any
    def __init__(self, name, format: Optional[Any] = ..., format_source: Optional[Any] = ..., metadata_source: Optional[Any] = ..., parent: Optional[Any] = ..., label: Optional[Any] = ..., filters: Optional[Any] = ..., actions: Optional[Any] = ..., hidden: bool = ..., implicit: bool = ...) -> None: ...
    def __len__(self): ...
    def __getitem__(self, index): ...
    def __iter__(self): ...
    def to_dict(self, view: str = ..., value_mapper: Optional[Any] = ..., app: Optional[Any] = ...): ...

class ToolOutputCollection(ToolOutputBase):
    collection = ...  # type: bool
    default_format = ...  # type: Any
    structure = ...  # type: Any
    outputs = ...  # type: Any
    inherit_format = ...  # type: Any
    inherit_metadata = ...  # type: Any
    metadata_source = ...  # type: Any
    format_source = ...  # type: Any
    change_format = ...  # type: Any
    def __init__(self, name, structure, label: Optional[Any] = ..., filters: Optional[Any] = ..., hidden: bool = ..., default_format: str = ..., default_format_source: Optional[Any] = ..., default_metadata_source: Optional[Any] = ..., inherit_format: bool = ..., inherit_metadata: bool = ...) -> None: ...
    def known_outputs(self, inputs, type_registry): ...
    @property
    def dynamic_structure(self): ...
    @property
    def dataset_collector_descriptions(self): ...

class ToolOutputCollectionStructure:
    collection_type = ...  # type: Any
    collection_type_source = ...  # type: Any
    structured_like = ...  # type: Any
    dataset_collector_descriptions = ...  # type: Any
    dynamic = ...  # type: Any
    def __init__(self, collection_type, collection_type_source, structured_like, dataset_collector_descriptions) -> None: ...

class ToolOutputCollectionPart:
    output_collection_def = ...  # type: Any
    element_identifier = ...  # type: Any
    output_def = ...  # type: Any
    parent_ids = ...  # type: Any
    def __init__(self, output_collection_def, element_identifier, output_def, parent_ids: Any = ...) -> None: ...
    @property
    def effective_output_name(self): ...
    @staticmethod
    def is_named_collection_part_name(name): ...
    @staticmethod
    def split_output_name(name): ...



# Stubs for networkx.algorithms.flow.edmondskarp (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.flow.utils import *
from typing import Any, Optional

def edmonds_karp(G, s, t, capacity: str = ..., residual: Optional[Any] = ..., value_only: bool = ..., cutoff: Optional[Any] = ...): ...



# Stubs for galaxy.tools.linters.command (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def lint_command(tool_xml, lint_ctx): ...
def get_command(tool_xml): ...



"""Tests of satisfying SoftwareRequirement via dependencies."""
import os
from pathlib import Path
from shutil import which
from types import ModuleType
from typing import Optional

import pytest

from .util import get_data, get_main_output, get_tool_env, needs_docker

deps = None  # type: Optional[ModuleType]
try:
    from galaxy.tool_util import deps
except ImportError:
    pass


@needs_docker
@pytest.mark.skipif(not deps, reason="galaxy-tool-util is not installed")
def test_biocontainers() -> None:
    wflow = get_data("tests/seqtk_seq.cwl")
    job = get_data("tests/seqtk_seq_job.json")
    error_code, _, _ = get_main_output(["--beta-use-biocontainers", wflow, job])

    assert error_code == 0


@pytest.mark.skipif(not deps, reason="galaxy-tool-util is not installed")
def test_bioconda() -> None:
    wflow = get_data("tests/seqtk_seq.cwl")
    job = get_data("tests/seqtk_seq_job.json")
    error_code, _, stderr = get_main_output(
        ["--beta-conda-dependencies", "--debug", wflow, job]
    )

    assert error_code == 0, stderr


@pytest.mark.skipif(not deps, reason="galaxy-tool-util is not installed")
@pytest.mark.skipif(not which("modulecmd"), reason="modulecmd not installed")
def test_modules(monkeypatch: pytest.MonkeyPatch) -> None:
    """Do a basic smoke test using environment modules to satisfy a SoftwareRequirement."""
    wflow = get_data("tests/random_lines.cwl")
    job = get_data("tests/random_lines_job.json")
    monkeypatch.setenv(
        "MODULEPATH", os.path.join(os.getcwd(), "tests/test_deps_env/modulefiles")
    )
    error_code, _, stderr = get_main_output(
        [
            "--beta-dependency-resolvers-configuration",
            "tests/test_deps_env_modules_resolvers_conf.yml",
            "--debug",
            wflow,
            job,
        ]
    )

    assert error_code == 0, stderr


@pytest.mark.skipif(not deps, reason="galaxy-tool-util is not installed")
@pytest.mark.skipif(not which("modulecmd"), reason="modulecmd not installed")
def test_modules_environment(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """
    Check that the environment variables set by a module are being propagated correctly.

    Do so by  by running `env` as the tool and parsing its output.
    """
    monkeypatch.setenv(
        "MODULEPATH", os.path.join(os.getcwd(), "tests/test_deps_env/modulefiles")
    )
    tool_env = get_tool_env(
        tmp_path,
        [
            "--beta-dependency-resolvers-configuration",
            get_data("tests/test_deps_env_modules_resolvers_conf.yml"),
        ],
        get_data("tests/env_with_software_req.yml"),
    )

    assert tool_env["TEST_VAR_MODULE"] == "environment variable ends in space "
    tool_path = tool_env["PATH"].split(":")
    assert get_data("tests/test_deps_env/random-lines/1.0/scripts") in tool_path



import pytest

from cwltool.cuda import cuda_version_and_device_count
from cwltool.main import main

from .util import get_data, needs_docker, needs_singularity_3_or_newer

cuda_version = cuda_version_and_device_count()


@needs_docker
@pytest.mark.skipif(
    cuda_version[0] == "", reason="nvidia-smi required for CUDA not detected"
)
def test_cuda_docker() -> None:
    params = [
        "--enable-ext",
        get_data("tests/wf/nvidia-smi-container.cwl"),
    ]
    assert main(params) == 0


@needs_singularity_3_or_newer
@pytest.mark.skipif(
    cuda_version[0] == "", reason="nvidia-smi required for CUDA not detected"
)
def test_cuda_singularity() -> None:
    params = [
        "--enable-ext",
        "--singularity",
        get_data("tests/wf/nvidia-smi-container.cwl"),
    ]
    assert main(params) == 0


@pytest.mark.skipif(
    cuda_version[0] == "", reason="nvidia-smi required for CUDA not detected"
)
def test_cuda_no_container() -> None:
    params = [
        "--enable-ext",
        "--singularity",
        get_data("tests/wf/nvidia-smi.cwl"),
    ]
    assert main(params) == 0



# Stubs for galaxy.jobs.metrics.instrumenters (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ...metrics import formatting as formatting

INSTRUMENT_FILE_PREFIX = ...  # type: str

class InstrumentPlugin:
    formatter = ...  # type: Any
    @property
    def plugin_type(self): ...
    def pre_execute_instrument(self, job_directory): ...
    def post_execute_instrument(self, job_directory): ...
    def job_properties(self, job_id, job_directory): ...



# Stubs for galaxy.util.submodules (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

log = ...  # type: Any

def submodules(module): ...



# Stubs for networkx.algorithms.centrality.current_flow_betweenness (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.centrality.flow_matrix import *
from typing import Any, Optional

def approximate_current_flow_betweenness_centrality(G, normalized: bool = ..., weight: Optional[Any] = ..., dtype: Any = ..., solver: str = ..., epsilon: float = ..., kmax: int = ...): ...
def current_flow_betweenness_centrality(G, normalized: bool = ..., weight: Optional[Any] = ..., dtype: Any = ..., solver: str = ...): ...
def edge_current_flow_betweenness_centrality(G, normalized: bool = ..., weight: Optional[Any] = ..., dtype: Any = ..., solver: str = ...): ...



import argparse
from typing import Any, Optional, Tuple, List, Generator, Dict

def find_locale_dir(): ...

TRANSLATION_CATALOG: Any
MODULE_NAME: Any
LOGGER: Any
VERSION: Any
PROJECT_URL: str
STANDARD_BAG_INFO_HEADERS: Any
CHECKSUM_ALGOS: Any
DEFAULT_CHECKSUMS: Any
HASH_BLOCK_SIZE: Any
open_text_file: Any
UNICODE_BYTE_ORDER_MARK: str

def make_bag(bag_dir: Any, bag_info: Optional[Any] = ..., processes: int = ..., checksums: Optional[Any] = ..., checksum: Optional[Any] = ..., encoding: str = ...): ...

class Bag:
    valid_files: Any = ...
    valid_directories: Any = ...
    tags: Any = ...
    info: Dict[str, str] = ...
    entries: Any = ...
    normalized_filesystem_names: Any = ...
    normalized_manifest_names: Any = ...
    algorithms: Any = ...
    tag_file_name: Any = ...
    path: Any = ...
    def __init__(self, path: Optional[Any] = ...) -> None: ...
    @property
    def algs(self): ...
    @property
    def version(self): ...
    def manifest_files(self) -> None: ...
    def tagmanifest_files(self) -> None: ...
    def compare_manifests_with_fs(self) -> Tuple[List[str], List[str]]: ...
    def compare_fetch_with_fs(self): ...
    def payload_files(self) -> None: ...
    def payload_entries(self): ...
    def save(self, processes: int = ..., manifests: bool = ...) -> None: ...
    def tagfile_entries(self): ...
    def missing_optional_tagfiles(self) -> Generator[str, None, None]: ...
    def fetch_entries(self) -> None: ...
    def files_to_be_fetched(self) -> None: ...
    def has_oxum(self) -> bool: ...
    def validate(self, processes: int = ..., fast: bool = ..., completeness_only: bool = ...): ...
    def is_valid(self, fast: bool = ..., completeness_only: bool = ...): ...
    def validate_fetch(self) -> None: ...

class BagError(Exception): ...

class BagValidationError(BagError):
    message: Any = ...
    details: Any = ...
    def __init__(self, message: Any, details: Optional[Any] = ...) -> None: ...

class ManifestErrorDetail(BagError):
    path: Any = ...
    def __init__(self, path: Any) -> None: ...

class ChecksumMismatch(ManifestErrorDetail):
    path: Any = ...
    algorithm: Any = ...
    expected: Any = ...
    found: Any = ...
    def __init__(self, path: Any, algorithm: Optional[Any] = ..., expected: Optional[Any] = ..., found: Optional[Any] = ...) -> None: ...

class FileMissing(ManifestErrorDetail): ...
class UnexpectedFile(ManifestErrorDetail): ...

class FileNormalizationConflict(BagError):
    file_a: Any = ...
    file_b: Any = ...
    def __init__(self, file_a: Any, file_b: Any) -> None: ...

def posix_multiprocessing_worker_initializer() -> None: ...
def normalize_unicode_py3(s: Any): ...
def normalize_unicode_py2(s: Any): ...
normalize_unicode = normalize_unicode_py3

def build_unicode_normalized_lookup_dict(filenames: Any): ...
def get_hashers(algorithms: Any): ...
def make_manifests(data_dir: Any, processes: Any, algorithms: Any = ..., encoding: str = ...): ...
def generate_manifest_lines(filename: Any, algorithms: Any = ...): ...
def force_unicode_py2(s: Any): ...
force_unicode = str

class BagArgumentParser(argparse.ArgumentParser):
    bag_info: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class BagHeaderAction(argparse.Action):
    def __call__(self, parser: Any, _: Any, values: Any, option_string: Optional[Any] = ...) -> None: ...

def main() -> None: ...



# Stubs for networkx.algorithms.flow.shortestaugmentingpath (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .utils import *
from typing import Any, Optional

def shortest_augmenting_path(G, s, t, capacity: str = ..., residual: Optional[Any] = ..., value_only: bool = ..., two_phase: bool = ..., cutoff: Optional[Any] = ...): ...



# Stubs for galaxy.util.properties (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from six.moves.configparser import ConfigParser

def find_config_file(default, old_default, explicit, cwd: Optional[Any] = ...): ...
def load_app_properties(kwds: Any = ..., ini_file: Optional[Any] = ..., ini_section: str = ..., config_prefix: str = ...): ...

class NicerConfigParser(ConfigParser):
    filename = ...  # type: Any
    def __init__(self, filename, *args, **kw) -> None: ...
    read_file = ...  # type: Any
    def defaults(self): ...
    class InterpolateWrapper:
        def __init__(self, original) -> None: ...
        def __getattr__(self, name): ...
        def before_get(self, parser, section, option, value, defaults): ...



# Stubs for galaxy.tools.locations (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class ToolLocationResolver:
    def scheme(self): ...
    def get_tool_source_path(self, uri_like): ...



# Stubs for galaxy.objectstore (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

object_session = ...  # type: Any
NO_SESSION_ERROR_MESSAGE = ...  # type: str
log = ...  # type: Any

class ObjectStore:
    running = ...  # type: bool
    extra_dirs = ...  # type: Any
    config = ...  # type: Any
    check_old_style = ...  # type: Any
    def __init__(self, config, **kwargs) -> None: ...
    def shutdown(self): ...
    def exists(self, obj, base_dir: Optional[Any] = ..., dir_only: bool = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ...): ...
    def file_ready(self, obj, base_dir: Optional[Any] = ..., dir_only: bool = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def create(self, obj, base_dir: Optional[Any] = ..., dir_only: bool = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def empty(self, obj, base_dir: Optional[Any] = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def size(self, obj, extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def delete(self, obj, entire_dir: bool = ..., base_dir: Optional[Any] = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def get_data(self, obj, start: int = ..., count: int = ..., base_dir: Optional[Any] = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def get_filename(self, obj, base_dir: Optional[Any] = ..., dir_only: bool = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def update_from_file(self, obj, base_dir: Optional[Any] = ..., extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ..., file_name: Optional[Any] = ..., create: bool = ...): ...
    def get_object_url(self, obj, extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ..., obj_dir: bool = ...): ...
    def get_store_usage_percent(self): ...

class DiskObjectStore(ObjectStore):
    file_path = ...  # type: Any
    def __init__(self, config, config_xml: Optional[Any] = ..., file_path: Optional[Any] = ..., extra_dirs: Optional[Any] = ...) -> None: ...
    def exists(self, obj, **kwargs): ...
    def create(self, obj, **kwargs): ...
    def empty(self, obj, **kwargs): ...
    def size(self, obj, **kwargs): ...
    def delete(self, obj, entire_dir: bool = ..., **kwargs): ...
    def get_data(self, obj, start: int = ..., count: int = ..., **kwargs): ...
    def get_filename(self, obj, **kwargs): ...
    def update_from_file(self, obj, file_name: Optional[Any] = ..., create: bool = ..., **kwargs): ...
    def get_object_url(self, obj, **kwargs): ...
    def get_store_usage_percent(self): ...

class NestedObjectStore(ObjectStore):
    backends = ...  # type: Any
    def __init__(self, config, config_xml: Optional[Any] = ...) -> None: ...
    def shutdown(self): ...
    def exists(self, obj, **kwargs): ...
    def file_ready(self, obj, **kwargs): ...
    def create(self, obj, **kwargs): ...
    def empty(self, obj, **kwargs): ...
    def size(self, obj, **kwargs): ...
    def delete(self, obj, **kwargs): ...
    def get_data(self, obj, **kwargs): ...
    def get_filename(self, obj, **kwargs): ...
    def update_from_file(self, obj, **kwargs): ...
    def get_object_url(self, obj, **kwargs): ...

class DistributedObjectStore(NestedObjectStore):
    distributed_config = ...  # type: Any
    backends = ...  # type: Any
    weighted_backend_ids = ...  # type: Any
    original_weighted_backend_ids = ...  # type: Any
    max_percent_full = ...  # type: Any
    global_max_percent_full = ...  # type: float
    sleeper = ...  # type: Any
    filesystem_monitor_thread = ...  # type: Any
    def __init__(self, config, config_xml: Optional[Any] = ..., fsmon: bool = ...) -> None: ...
    def shutdown(self): ...
    def create(self, obj, **kwargs): ...

class HierarchicalObjectStore(NestedObjectStore):
    backends = ...  # type: Any
    def __init__(self, config, config_xml: Optional[Any] = ..., fsmon: bool = ...) -> None: ...
    def exists(self, obj, **kwargs): ...
    def create(self, obj, **kwargs): ...

def build_object_store_from_config(config, fsmon: bool = ..., config_xml: Optional[Any] = ...): ...
def local_extra_dirs(func): ...
def convert_bytes(bytes): ...



# Stubs for galaxy.tools.parser.factory (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from galaxy.tools.loader import load_tool as load_tool_xml

def get_tool_source(config_file: Optional[Any] = ..., xml_tree: Optional[Any] = ..., enable_beta_formats: bool = ..., tool_location_fetcher: Optional[Any] = ...): ...
def get_input_source(content): ...



# Stubs for networkx.algorithms.connectivity.connectivity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.flow import edmonds_karp
from typing import Any, Optional

default_flow_func = edmonds_karp

def local_node_connectivity(G, s, t, flow_func: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ..., cutoff: Optional[Any] = ...): ...
def node_connectivity(G, s: Optional[Any] = ..., t: Optional[Any] = ..., flow_func: Optional[Any] = ...): ...
def average_node_connectivity(G, flow_func: Optional[Any] = ...): ...
def all_pairs_node_connectivity(G, nbunch: Optional[Any] = ..., flow_func: Optional[Any] = ...): ...
def local_edge_connectivity(G, s, t, flow_func: Optional[Any] = ..., auxiliary: Optional[Any] = ..., residual: Optional[Any] = ..., cutoff: Optional[Any] = ...): ...
def edge_connectivity(G, s: Optional[Any] = ..., t: Optional[Any] = ..., flow_func: Optional[Any] = ..., cutoff: Optional[Any] = ...): ...



# Stubs for galaxy.util.bunch (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class Bunch:
    def __init__(self, **kwds) -> None: ...
    def dict(self): ...
    def get(self, key, default: Optional[Any] = ...): ...
    def __iter__(self): ...
    def items(self): ...
    def keys(self): ...
    def values(self): ...
    def __nonzero__(self): ...
    def __setitem__(self, k, v): ...
    def __contains__(self, item): ...



# Stubs for galaxy.util.plugin_config (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

yaml = ...  # type: Any

def plugins_dict(module, plugin_type_identifier): ...
def load_plugins(plugins_dict, plugin_source, extra_kwds: Any = ...): ...
def plugin_source_from_path(path): ...



# Stubs for galaxy.objectstore.s3_multipart_upload (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

boto = ...  # type: Any

def map_wrap(f): ...
def mp_from_ids(s3server, mp_id, mp_keyname, mp_bucketname): ...
def transfer_part(s3server, mp_id, mp_keyname, mp_bucketname, i, part): ...
def multipart_upload(s3server, bucket, s3_key_name, tarball, mb_size): ...
def multimap(cores: Optional[Any] = ...): ...



from typing import Any, Optional
from unittest import mock
from unittest.mock import MagicMock

from cwltool.loghandler import _logger
from cwltool.main import main

from .util import get_data


class MockResponse1:
    def __init__(
        self, json_data: Any, status_code: int, raise_for_status: Optional[bool] = None
    ) -> None:
        """Create a fake return object for requests.Session.head."""
        self.json_data = json_data
        self.status_code = status_code
        self.raise_for_status = mock.Mock()
        self.raise_for_status.side_effect = raise_for_status

    def json(self) -> Any:
        return self.json_data


def mocked_requests_head(*args: Any, **kwargs: Any) -> MockResponse1:
    return MockResponse1(None, 200)


class MockResponse2:
    def __init__(
        self, json_data: Any, status_code: int, raise_for_status: Optional[bool] = None
    ) -> None:
        """Create a fake return object for requests.Session.get."""
        self.json_data = json_data
        self.text = json_data
        self.status_code = status_code
        self.raise_for_status = mock.Mock()
        self.raise_for_status.side_effect = raise_for_status

    def json(self) -> Any:
        return self.json_data

    headers = {"content-type": "text/plain"}


def mocked_requests_get(*args: Any, **kwargs: Any) -> MockResponse2:
    if (
        args[0]
        == "https://dockstore.org/api/api/ga4gh/v2/tools/quay.io%2Fbriandoconnor%2Fdockstore-tool-md5sum/versions/1.0.4/CWL/files"
    ):
        return MockResponse2(
            [
                {"file_type": "CONTAINERFILE", "path": "Dockerfile"},
                {"file_type": "PRIMARY_DESCRIPTOR", "path": "Dockstore.cwl"},
                {"file_type": "TEST_FILE", "path": "test.json"},
            ],
            200,
        )
    elif (
        args[0]
        == "https://dockstore.org/api/api/ga4gh/v2/tools/quay.io%2Fbriandoconnor%2Fdockstore-tool-md5sum/versions/1.0.4/plain-CWL/descriptor/Dockstore.cwl"
    ):
        string = open(get_data("tests/trs/Dockstore.cwl")).read()
        return MockResponse2(string, 200)
    elif (
        args[0]
        == "https://dockstore.org/api/api/ga4gh/v2/tools/%23workflow%2Fgithub.com%2Fdockstore-testing%2Fmd5sum-checker/versions/develop/plain-CWL/descriptor/md5sum-tool.cwl"
    ):
        string = open(get_data("tests/trs/md5sum-tool.cwl")).read()
        return MockResponse2(string, 200)
    elif (
        args[0]
        == "https://dockstore.org/api/api/ga4gh/v2/tools/%23workflow%2Fgithub.com%2Fdockstore-testing%2Fmd5sum-checker/versions/develop/plain-CWL/descriptor/md5sum-workflow.cwl"
    ):
        string = open(get_data("tests/trs/md5sum-workflow.cwl")).read()
        return MockResponse2(string, 200)
    elif (
        args[0]
        == "https://dockstore.org/api/api/ga4gh/v2/tools/%23workflow%2Fgithub.com%2Fdockstore-testing%2Fmd5sum-checker/versions/develop/CWL/files"
    ):
        return MockResponse2(
            [
                {"file_type": "TEST_FILE", "path": "md5sum-input-cwl.json"},
                {"file_type": "SECONDARY_DESCRIPTOR", "path": "md5sum-tool.cwl"},
                {"file_type": "PRIMARY_DESCRIPTOR", "path": "md5sum-workflow.cwl"},
            ],
            200,
        )

    _logger.debug("A mocked call to TRS missed, target was %s", args[0])
    return MockResponse2(None, 404)


@mock.patch("requests.Session.head", side_effect=mocked_requests_head)
@mock.patch("requests.Session.get", side_effect=mocked_requests_get)
def test_tool_trs_template(mock_head: MagicMock, mock_get: MagicMock) -> None:
    params = [
        "--debug",
        "--make-template",
        r"quay.io/briandoconnor/dockstore-tool-md5sum:1.0.4",
    ]
    return_value = main(params)
    mock_head.assert_called()
    mock_get.assert_called()
    assert return_value == 0


@mock.patch("requests.Session.head", side_effect=mocked_requests_head)
@mock.patch("requests.Session.get", side_effect=mocked_requests_get)
def test_workflow_trs_template(mock_head: MagicMock, mock_get: MagicMock) -> None:
    params = [
        "--debug",
        "--make-template",
        r"#workflow/github.com/dockstore-testing/md5sum-checker:develop",
    ]
    return_value = main(params)
    mock_head.assert_called()
    mock_get.assert_called()
    assert return_value == 0



# Stubs for galaxy.tools.deps.containers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional, Text
from .container_resolvers.explicit import ExplicitContainerResolver as ExplicitContainerResolver
from .container_resolvers.mulled import BuildMulledDockerContainerResolver as BuildMulledDockerContainerResolver, BuildMulledSingularityContainerResolver as BuildMulledSingularityContainerResolver, CachedMulledDockerContainerResolver as CachedMulledDockerContainerResolver, CachedMulledSingularityContainerResolver as CachedMulledSingularityContainerResolver, MulledDockerContainerResolver as MulledDockerContainerResolver
from .requirements import ContainerDescription
from .requirements import DEFAULT_CONTAINER_RESOLVE_DEPENDENCIES as DEFAULT_CONTAINER_RESOLVE_DEPENDENCIES, DEFAULT_CONTAINER_SHELL as DEFAULT_CONTAINER_SHELL
from ..deps import docker_util as docker_util
from ..deps import singularity_util as singularity_util

log = ...  # type: Any
DOCKER_CONTAINER_TYPE = ...  # type: str
SINGULARITY_CONTAINER_TYPE = ...  # type: str
DEFAULT_CONTAINER_TYPE = ...  # type: Any
ALL_CONTAINER_TYPES = ...  # type: Any
LOAD_CACHED_IMAGE_COMMAND_TEMPLATE = ...  # type: str

class ContainerFinder:
    app_info = ...  # type: Any
    container_registry = ...  # type: Any
    def __init__(self, app_info) -> None: ...
    def find_best_container_description(self, enabled_container_types, tool_info) -> ContainerDescription: ...
    def find_container(self, tool_info, destination_info, job_info): ...

class NullContainerFinder:
    def find_container(self, tool_info, destination_info, job_info): ...

class ContainerRegistry:
    resolver_classes = ...  # type: Any
    enable_beta_mulled_containers = ...  # type: Any
    app_info = ...  # type: Any
    container_resolvers = ...  # type: Any
    def __init__(self, app_info) -> None: ...
    def find_best_container_description(self, enabled_container_types, tool_info) -> ContainerDescription: ...

class JobInfo:
    working_directory = ...  # type: Any
    job_directory = ...  # type: Any
    tool_directory = ...  # type: Any
    job_directory_type = ...  # type: Any
    def __init__(self, working_directory, tool_directory, job_directory, job_directory_type) -> None: ...

class Container:
    container_id = ...  # type: Any
    app_info = ...  # type: Any
    tool_info = ...  # type: Any
    destination_info = ...  # type: Any
    job_info = ...  # type: Any
    container_description = ...  # type: Any
    def __init__(self, container_id, app_info, tool_info, destination_info, job_info, container_description) -> None: ...
    @property
    def resolve_dependencies(self): ...
    @property
    def shell(self): ...
    def containerize_command(self, command): ...

def preprocess_volumes(volumes_raw_str, container_type): ...

class HasDockerLikeVolumes: ...

class DockerContainer(Container, HasDockerLikeVolumes):
    container_type = ...  # type: Any
    def containerize_command(self, command): ...

def docker_cache_path(cache_directory, container_id): ...

class SingularityContainer(Container, HasDockerLikeVolumes):
    container_type = ...  # type: Any
    def containerize_command(self, command): ...

CONTAINER_CLASSES = ...  # type: Any

class NullContainer:
    def __init__(self) -> None: ...
    def __bool__(self): ...
    __nonzero__ = ...  # type: Any

NULL_CONTAINER = ...  # type: Any



from urllib.parse import ParseResult
from typing import Any
from uuid import NAMESPACE_URL as NAMESPACE_URL

SCHEME: str

def is_arcp_uri(uri: Any): ...
def parse_arcp(uri: Any): ...
def urlparse(uri: Any): ...

class ARCPParseResult(ParseResult):
    def __init__(self, *args: Any) -> None: ...
    @property
    def prefix(self): ...
    @property
    def name(self): ...
    @property
    def uuid(self): ...
    @property
    def ni(self): ...
    def ni_uri(self, authority: str = ...): ...
    def nih_uri(self): ...
    def ni_well_known(self, base: str = ...): ...
    @property
    def hash(self): ...



# Stubs for galaxy.util.image_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from PIL import Image as PIL

PIL = ...  # type: Any
log = ...  # type: Any

def image_type(filename): ...
def check_image_type(filename, types): ...
def get_image_ext(file_path): ...



# Stubs for galaxy.tools.parser (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .factory import get_input_source as get_input_source, get_tool_source as get_tool_source
from .interface import ToolSource as ToolSource
from .output_objects import ToolOutputCollectionPart as ToolOutputCollectionPart



# Stubs for networkx.algorithms.centrality.katz (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def katz_centrality(G, alpha: float = ..., beta: float = ..., max_iter: int = ..., tol: float = ..., nstart: Optional[Any] = ..., normalized: bool = ..., weight: Optional[Any] = ...): ...
def katz_centrality_numpy(G, alpha: float = ..., beta: float = ..., normalized: bool = ..., weight: Optional[Any] = ...): ...



# Stubs for galaxy.tools.verify.asserts (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

log = ...  # type: Any
assertion_module_names = ...  # type: Any
assertion_modules = ...  # type: Any
full_assertion_module_name = ...  # type: Any
assertion_module = ...  # type: Any

def verify_assertions(data, assertion_description_list): ...
def verify_assertion(data, assertion_description): ...



# Stubs for galaxy.tools.deps.resolvers.galaxy_packages (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .resolver_mixins import UsesToolDependencyDirMixin
from ..resolvers import Dependency, DependencyResolver, ListableDependencyResolver, MappableDependencyResolver

class GalaxyPackageDependency(Dependency):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: str
    script = ...  # type: Any
    path = ...  # type: Any
    version = ...  # type: Any
    name = ...  # type: Any
    def __init__(self, script, path, version, name, exact: bool = ...) -> None: ...
    @property
    def exact(self): ...
    def shell_commands(self, requirement): ...

class ToolShedDependency(GalaxyPackageDependency):
    dependency_type = ...  # type: str

class BaseGalaxyPackageDependencyResolver(DependencyResolver, UsesToolDependencyDirMixin):
    dict_collection_visible_keys = ...  # type: Any
    dependency_type = ...  # type: Any
    versionless = ...  # type: Any
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def resolve(self, requirement, **kwds): ...

class GalaxyPackageDependencyResolver(BaseGalaxyPackageDependencyResolver, ListableDependencyResolver, MappableDependencyResolver):
    resolver_type = ...  # type: str
    def __init__(self, dependency_manager, **kwds) -> None: ...
    def resolve(self, requirement, **kwds): ...
    def list_dependencies(self): ...



# Stubs for galaxy.tools.cwl.representation (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

log = ...  # type: Any
NOT_PRESENT = ...  # type: Any
GALAXY_TO_CWL_TYPES = ...  # type: Any

def to_cwl_job(tool, param_dict, local_working_directory): ...
def to_galaxy_parameters(tool, as_dict): ...



# Stubs for prov.serializers.provn (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov.serializers import Serializer
from typing import Any

__email__: str
logger: Any

class ProvNSerializer(Serializer):
    def serialize(self, stream: Any, **kwargs: Any) -> None: ...
    def deserialize(self, stream: Any, **kwargs: Any) -> None: ...



# Stubs for galaxy.jobs.metrics.collectl.processes (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def generate_process_statistics(collectl_playback_cli, pid, statistics: Any = ...): ...

class CollectlProcessSummarizer:
    pid = ...  # type: Any
    statistics = ...  # type: Any
    columns_of_interest = ...  # type: Any
    tree_statistics = ...  # type: Any
    process_accum_statistics = ...  # type: Any
    interval_count = ...  # type: int
    def __init__(self, pid, statistics) -> None: ...
    def handle_interval(self, interval): ...
    def get_statistics(self): ...

class CollectlProcessInterval:
    rows = ...  # type: Any
    def __init__(self) -> None: ...
    def row_is_in(self, row): ...
    def add_row(self, row): ...



# Stubs for networkx.algorithms.link_analysis.hits_alg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def hits(G, max_iter: int = ..., tol: float = ..., nstart: Optional[Any] = ..., normalized: bool = ...): ...
def authority_matrix(G, nodelist: Optional[Any] = ...): ...
def hub_matrix(G, nodelist: Optional[Any] = ...): ...
def hits_numpy(G, normalized: bool = ...): ...
def hits_scipy(G, max_iter: int = ..., tol: float = ..., normalized: bool = ...): ...



# Stubs for galaxy.tools.deps.container_resolvers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from galaxy.util.dictifiable import Dictifiable

class ContainerResolver(Dictifiable):
    dict_collection_visible_keys = ...  # type: Any
    app_info = ...  # type: Any
    resolver_kwds = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., **kwds) -> None: ...
    def resolve(self, tool_info): ...
    def resolver_type(self): ...



# Stubs for galaxy.tools.linters.xml_order (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

TAG_ORDER = ...  # type: Any
DATASOURCE_TAG_ORDER = ...  # type: Any

def lint_xml_order(tool_xml, lint_ctx): ...



# Stubs for networkx.algorithms.assortativity.correlation (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def degree_assortativity_coefficient(G, x: str = ..., y: str = ..., weight: Optional[Any] = ..., nodes: Optional[Any] = ...): ...
def degree_pearson_correlation_coefficient(G, x: str = ..., y: str = ..., weight: Optional[Any] = ..., nodes: Optional[Any] = ...): ...
def attribute_assortativity_coefficient(G, attribute, nodes: Optional[Any] = ...): ...
def numeric_assortativity_coefficient(G, attribute, nodes: Optional[Any] = ...): ...



# Stubs for galaxy.tools.parser.cwl (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .interface import PageSource as PageSource
from .interface import PagesSource as PagesSource
from .interface import ToolSource as ToolSource
from .interface import ToolStdioExitCode as ToolStdioExitCode
from .output_actions import ToolOutputActionGroup as ToolOutputActionGroup
from .output_objects import ToolOutput as ToolOutput
from .yaml import YamlInputSource as YamlInputSource

log = ...  # type: Any

class CwlToolSource(ToolSource):
    def __init__(self, tool_file, strict_cwl_validation: bool = ...) -> None: ...
    @property
    def tool_proxy(self): ...
    def parse_tool_type(self): ...
    def parse_id(self): ...
    def parse_name(self): ...
    def parse_command(self): ...
    def parse_environment_variables(self): ...
    def parse_edam_operations(self): ...
    def parse_edam_topics(self): ...
    def parse_help(self): ...
    def parse_sanitize(self): ...
    def parse_strict_shell(self): ...
    def parse_stdio(self): ...
    def parse_interpreter(self): ...
    def parse_version(self): ...
    def parse_description(self): ...
    def parse_input_pages(self): ...
    def parse_outputs(self, tool): ...
    def parse_requirements_and_containers(self): ...
    def parse_profile(self): ...

class CwlPageSource(PageSource):
    def __init__(self, tool_proxy) -> None: ...
    def parse_input_sources(self): ...



# Stubs for galaxy.tools.deps.resolvers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from galaxy.util.dictifiable import Dictifiable
from ..requirements import ToolRequirement as ToolRequirement

class DependencyResolver(Dictifiable):
    dict_collection_visible_keys = ...  # type: Any
    disabled = ...  # type: bool
    resolves_simple_dependencies = ...  # type: bool
    can_uninstall_dependencies = ...  # type: bool
    config_options = ...  # type: Any
    def resolve(self, requirement, **kwds): ...

class MultipleDependencyResolver:
    def resolve_all(self, requirements, **kwds): ...

class ListableDependencyResolver:
    def list_dependencies(self): ...

class MappableDependencyResolver: ...

FROM_UNVERSIONED = ...  # type: Any

class RequirementMapping:
    from_name = ...  # type: Any
    from_version = ...  # type: Any
    to_name = ...  # type: Any
    to_version = ...  # type: Any
    def __init__(self, from_name, from_version, to_name, to_version) -> None: ...
    def matches_requirement(self, requirement): ...
    def apply(self, requirement): ...
    @staticmethod
    def from_dict(raw_mapping): ...

class SpecificationAwareDependencyResolver: ...
class SpecificationPatternDependencyResolver: ...

class InstallableDependencyResolver:
    def install_dependency(self, name, version, type, **kwds): ...

class Dependency(Dictifiable):
    dict_collection_visible_keys = ...  # type: Any
    cacheable = ...  # type: bool
    def shell_commands(self, requirement): ...
    def exact(self): ...
    @property
    def resolver_msg(self): ...

class NullDependency(Dependency):
    dependency_type = ...  # type: Any
    exact = ...  # type: bool
    version = ...  # type: Any
    name = ...  # type: Any
    def __init__(self, version: Optional[Any] = ..., name: Optional[Any] = ...) -> None: ...
    @property
    def resolver_msg(self): ...
    def shell_commands(self, requirement): ...

class DependencyException(Exception): ...



# Stubs for galaxy.objectstore.azure_blob (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..objectstore import convert_bytes as convert_bytes, ObjectStore as ObjectStore

BlockBlobService = ...  # type: Any
NO_BLOBSERVICE_ERROR_MESSAGE = ...  # type: str
log = ...  # type: Any

class AzureBlobObjectStore(ObjectStore):
    staging_path = ...  # type: Any
    transfer_progress = ...  # type: int
    cache_size = ...  # type: Any
    sleeper = ...  # type: Any
    cache_monitor_thread = ...  # type: Any
    def __init__(self, config, config_xml) -> None: ...
    def exists(self, obj, **kwargs): ...
    def file_ready(self, obj, **kwargs): ...
    def create(self, obj, **kwargs): ...
    def empty(self, obj, **kwargs): ...
    def size(self, obj, **kwargs): ...
    def delete(self, obj, entire_dir: bool = ..., **kwargs): ...
    def get_data(self, obj, start: int = ..., count: int = ..., **kwargs): ...
    def get_filename(self, obj, **kwargs): ...
    def update_from_file(self, obj, file_name: Optional[Any] = ..., create: bool = ..., **kwargs): ...
    def get_object_url(self, obj, **kwargs): ...
    def get_store_usage_percent(self): ...



# Stubs for galaxy.tools.deps.container_resolvers.mulled (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..container_resolvers import ContainerResolver
from collections import namedtuple

CachedMulledImageSingleTarget = namedtuple('CachedMulledImageSingleTarget', ['package_name', 'version', 'build', 'image_identifier'])

CachedV1MulledImageMultiTarget = namedtuple('CachedV1MulledImageMultiTarget', ['hash', 'build', 'image_identifier'])

CachedV2MulledImageMultiTarget = namedtuple('CachedV2MulledImageMultiTarget', ['package_hash', 'version_hash', 'build', 'image_identifier'])

class CachedMulledDockerContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    container_type = ...  # type: str
    namespace = ...  # type: Any
    hash_func = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., namespace: Optional[Any] = ..., hash_func: str = ...) -> None: ...
    def resolve(self, enabled_container_types, tool_info): ...

class CachedMulledSingularityContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    container_type = ...  # type: str
    cache_directory = ...  # type: Any
    hash_func = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., hash_func: str = ...) -> None: ...
    def resolve(self, enabled_container_types, tool_info): ...

class MulledDockerContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    container_type = ...  # type: str
    namespace = ...  # type: Any
    hash_func = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., namespace: str = ..., hash_func: str = ...) -> None: ...
    def resolve(self, enabled_container_types, tool_info): ...

class BuildMulledDockerContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    container_type = ...  # type: str
    namespace = ...  # type: Any
    hash_func = ...  # type: Any
    auto_init = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., namespace: str = ..., hash_func: str = ..., **kwds) -> None: ...
    def resolve(self, enabled_container_types, tool_info): ...

class BuildMulledSingularityContainerResolver(ContainerResolver):
    resolver_type = ...  # type: str
    container_type = ...  # type: str
    cache_directory = ...  # type: Any
    hash_func = ...  # type: Any
    auto_init = ...  # type: Any
    def __init__(self, app_info: Optional[Any] = ..., hash_func: str = ..., **kwds) -> None: ...
    def resolve(self, enabled_container_types, tool_info): ...



# Stubs for galaxy.exceptions.error_codes (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

UNKNOWN_ERROR_MESSAGE = ...  # type: str

class ErrorCode:
    code = ...  # type: Any
    default_error_message = ...  # type: Any
    def __init__(self, code, default_error_message) -> None: ...
    def __int__(self): ...

error_codes_json = ...  # type: Any
name = ...  # type: Any
error_code_obj = ...  # type: Any



# Stubs for galaxy.tools.deps.brew_exts (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

WHITESPACE_PATTERN = ...  # type: Any
DESCRIPTION = ...  # type: str
DEFAULT_HOMEBREW_ROOT = ...  # type: str
NO_BREW_ERROR_MESSAGE = ...  # type: str
CANNOT_DETERMINE_TAP_ERROR_MESSAGE = ...  # type: str
VERBOSE = ...  # type: bool
RELAXED = ...  # type: bool
BREW_ARGS = ...  # type: Any

class BrewContext:
    homebrew_prefix = ...  # type: Any
    homebrew_cellar = ...  # type: Any
    def __init__(self, args: Optional[Any] = ...) -> None: ...

class RecipeContext:
    @staticmethod
    def from_args(args, brew_context: Optional[Any] = ...): ...
    recipe = ...  # type: Any
    version = ...  # type: Any
    brew_context = ...  # type: Any
    def __init__(self, recipe, version, brew_context: Optional[Any] = ...) -> None: ...
    @property
    def cellar_path(self): ...
    @property
    def tap_path(self): ...

def main(): ...

class CommandLineException(Exception):
    command = ...  # type: Any
    stdout = ...  # type: Any
    stderr = ...  # type: Any
    message = ...  # type: Any
    def __init__(self, command, stdout, stderr) -> None: ...

def versioned_install(recipe_context, package: Optional[Any] = ..., version: Optional[Any] = ..., installed_deps: Any = ...): ...
def commit_for_version(recipe_context, package, version): ...
def print_versioned_deps(recipe_context, recipe, version): ...
def load_versioned_deps(cellar_path, relaxed: Optional[Any] = ...): ...
def unversioned_install(package): ...
def attempt_unlink_all(package, deps): ...
def attempt_unlink(package): ...
def brew_execute(args, env: Optional[Any] = ...): ...
def build_env_statements_from_recipe_context(recipe_context, **kwds): ...
def build_env_statements(cellar_root, cellar_path, relaxed: Optional[Any] = ..., custom_only: bool = ...): ...
def build_env_actions(deps, cellar_root, cellar_path, relaxed: Optional[Any] = ..., custom_only: bool = ...): ...

class EnvAction:
    variable = ...  # type: Any
    action = ...  # type: Any
    value = ...  # type: Any
    def __init__(self, keg_root, action_description) -> None: ...
    @staticmethod
    def build_env(env_actions): ...
    def modify_environ(self, environ): ...
    def to_statements(self): ...

def brew_head_at_version(recipe_context, package, version): ...
def brew_head_at_commit(commit, tap_path): ...
def git_execute(args): ...
def execute(cmds, env: Optional[Any] = ...): ...
def brew_deps(package): ...
def brew_info(recipe): ...
def extended_brew_info(recipe): ...
def brew_versions_info(package, tap_path): ...
def recipe_cellar_path(cellar_path, recipe, version): ...
def ensure_brew_on_path(args): ...
def which(file): ...



# Stubs for galaxy.util.lazy_process (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class LazyProcess:
    command_and_args = ...  # type: Any
    thread_lock = ...  # type: Any
    allow_process_request = ...  # type: bool
    process = ...  # type: Any
    def __init__(self, command_and_args) -> None: ...
    def start_process(self): ...
    def shutdown(self): ...
    @property
    def running(self): ...

class NoOpLazyProcess:
    def start_process(self): ...
    def shutdown(self): ...
    @property
    def running(self): ...



# Stubs for galaxy.util.hash_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import hashlib as hashlib

sha1 = ...  # type: Any
sha = ...  # type: Any
md5 = ...  # type: Any

def new_secure_hash(text_type: Optional[Any] = ...): ...
def hmac_new(key, value): ...
def is_hashable(value): ...



# Stubs for galaxy.util.object_wrapper (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from six.moves import copyreg as copy_reg
from galaxy.util import sanitize_lists_to_string as _sanitize_lists_to_string

NoneType = ...  # type: Any
NotImplementedType = ...  # type: Any
EllipsisType = ...  # type: Any
XRangeType = ...  # type: Any
SliceType = ...  # type: Any
BufferType = ...  # type: Any
DictProxyType = ...  # type: Any
log = ...  # type: Any
__CALLABLE_TYPES__ = ...  # type: Any
__WRAP_NO_SUBCLASS__ = ...  # type: Any
__DONT_SANITIZE_TYPES__ = ...  # type: Any
__DONT_WRAP_TYPES__ = ...  # type: Any
__WRAP_SEQUENCES__ = ...  # type: Any
__WRAP_SETS__ = ...  # type: Any
__WRAP_MAPPINGS__ = ...  # type: Any
VALID_CHARACTERS = ...  # type: Any
CHARACTER_MAP = ...  # type: Any
INVALID_CHARACTER = ...  # type: str

def coerce(x, y): ...
def cmp(x, y): ...
def sanitize_lists_to_string(values, valid_characters: Any = ..., character_map: Any = ..., invalid_character: Any = ...): ...
def wrap_with_safe_string(value, no_wrap_classes: Optional[Any] = ...): ...

class SafeStringWrapper:
    __UNSANITIZED_ATTRIBUTE_NAME__ = ...  # type: str
    __NO_WRAP_NAMES__ = ...  # type: Any
    def __new__(cls, *arg, **kwd): ...
    unsanitized = ...  # type: Any
    __safe_string_wrapper_function__ = ...  # type: Any
    def __init__(self, value, safe_string_wrapper_function: Any = ...) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __cmp__(self, other): ...
    def __hash__(self): ...
    def __bool__(self): ...
    __nonzero__ = ...  # type: Any
    def __getattr__(self, name): ...
    def __setattr__(self, name, value): ...
    def __delattr__(self, name): ...
    def __getattribute__(self, name): ...
    def __len__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def __delitem__(self, key): ...
    def __iter__(self): ...
    def __contains__(self, item): ...
    def __getslice__(self, i, j): ...
    def __setslice__(self, i, j, value): ...
    def __delslice__(self, i, j): ...
    def __add__(self, other): ...
    def __sub__(self, other): ...
    def __mul__(self, other): ...
    def __floordiv__(self, other): ...
    def __mod__(self, other): ...
    def __divmod__(self, other): ...
    def __pow__(self, *other): ...
    def __lshift__(self, other): ...
    def __rshift__(self, other): ...
    def __and__(self, other): ...
    def __xor__(self, other): ...
    def __or__(self, other): ...
    def __div__(self, other): ...
    def __truediv__(self, other): ...
    def __rpow__(self, other): ...
    def __neg__(self): ...
    def __pos__(self): ...
    def __abs__(self): ...
    def __invert__(self): ...
    def __complex__(self): ...
    def __int__(self): ...
    def __float__(self): ...
    def __oct__(self): ...
    def __hex__(self): ...
    def __index__(self): ...
    def __coerce__(self, other): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

class CallableSafeStringWrapper(SafeStringWrapper):
    def __call__(self, *args, **kwds): ...

def pickle_SafeStringWrapper(safe_object): ...



# Stubs for networkx.algorithms.centrality.closeness (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def closeness_centrality(G, u: Optional[Any] = ..., distance: Optional[Any] = ..., wf_improved: bool = ..., reverse: bool = ...): ...



# Stubs for galaxy.tools.lint (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .parser import get_tool_source as get_tool_source

LEVEL_ALL = ...  # type: str
LEVEL_WARN = ...  # type: str
LEVEL_ERROR = ...  # type: str

def lint_tool_source(tool_source, level: Any = ..., fail_level: Any = ..., extra_modules: Any = ..., skip_types: Any = ...): ...
def lint_xml(tool_xml, level: Any = ..., fail_level: Any = ..., extra_modules: Any = ..., skip_types: Any = ...): ...
def lint_tool_source_with(lint_context, tool_source, extra_modules: Any = ...): ...
def lint_xml_with(lint_context, tool_xml, extra_modules: Any = ...): ...

class LintContext:
    skip_types = ...  # type: Any
    level = ...  # type: Any
    found_errors = ...  # type: bool
    found_warns = ...  # type: bool
    def __init__(self, level, skip_types: Any = ...) -> None: ...
    printed_linter_info = ...  # type: bool
    valid_messages = ...  # type: Any
    info_messages = ...  # type: Any
    warn_messages = ...  # type: Any
    error_messages = ...  # type: Any
    def lint(self, name, lint_func, lint_target): ...
    def valid(self, message, *args): ...
    def info(self, message, *args): ...
    def error(self, message, *args): ...
    def warn(self, message, *args): ...
    def failed(self, fail_level): ...



# Stubs for galaxy.tools.verify.asserts.tabular (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def get_first_line(output): ...
def assert_has_n_columns(output, n, sep: str = ...): ...



# Stubs for galaxy.jobs.metrics (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..metrics import formatting as formatting

log = ...  # type: Any
DEFAULT_FORMATTER = ...  # type: Any

class JobMetrics:
    plugin_classes = ...  # type: Any
    default_job_instrumenter = ...  # type: Any
    job_instrumenters = ...  # type: Any
    def __init__(self, conf_file: Optional[Any] = ..., **kwargs) -> None: ...
    def format(self, plugin, key, value): ...
    def set_destination_conf_file(self, destination_id, conf_file): ...
    def set_destination_conf_element(self, destination_id, element): ...
    def set_destination_instrumenter(self, destination_id, job_instrumenter: Optional[Any] = ...): ...
    def collect_properties(self, destination_id, job_id, job_directory): ...

class NullJobInstrumenter:
    def pre_execute_commands(self, job_directory): ...
    def post_execute_commands(self, job_directory): ...
    def collect_properties(self, job_id, job_directory): ...

NULL_JOB_INSTRUMENTER = ...  # type: Any

class JobInstrumenter:
    extra_kwargs = ...  # type: Any
    plugin_classes = ...  # type: Any
    plugins = ...  # type: Any
    def __init__(self, plugin_classes, plugins_source, **kwargs) -> None: ...
    def pre_execute_commands(self, job_directory): ...
    def post_execute_commands(self, job_directory): ...
    def collect_properties(self, job_id, job_directory): ...
    @staticmethod
    def from_file(plugin_classes, conf_file, **kwargs): ...



# Stubs for networkx.algorithms.link_analysis.pagerank_alg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def pagerank(G, alpha: float = ..., personalization: Optional[Any] = ..., max_iter: int = ..., tol: float = ..., nstart: Optional[Any] = ..., weight: str = ..., dangling: Optional[Any] = ...): ...
def google_matrix(G, alpha: float = ..., personalization: Optional[Any] = ..., nodelist: Optional[Any] = ..., weight: str = ..., dangling: Optional[Any] = ...): ...
def pagerank_numpy(G, alpha: float = ..., personalization: Optional[Any] = ..., weight: str = ..., dangling: Optional[Any] = ...): ...
def pagerank_scipy(G, alpha: float = ..., personalization: Optional[Any] = ..., max_iter: int = ..., tol: float = ..., weight: str = ..., dangling: Optional[Any] = ...): ...



# Stubs for galaxy.util.topsort (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from galaxy.util.odict import odict as OrderedDict

class CycleError(Exception):
    preds = ...  # type: Any
    def __init__(self, sofar, numpreds, succs) -> None: ...
    def get_partial(self): ...
    def get_pred_counts(self): ...
    def get_succs(self): ...
    def get_elements(self): ...
    def get_pairlist(self): ...
    def get_preds(self): ...
    def pick_a_cycle(self): ...

def topsort(pairlist): ...
def topsort_levels(pairlist): ...



# Stubs for galaxy.util.expressions (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from collections import MutableMapping

class ExpressionContext(MutableMapping):
    dict = ...  # type: Any
    parent = ...  # type: Any
    def __init__(self, dict, parent: Optional[Any] = ...) -> None: ...
    def __delitem__(self, key): ...
    def __iter__(self): ...
    def __len__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def __contains__(self, key): ...
    def __nonzero__(self): ...



# Stubs for galaxy.exceptions (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..exceptions import error_codes as error_codes

class MessageException(Exception):
    status_code = ...  # type: int
    err_code = ...  # type: Any
    err_msg = ...  # type: Any
    type = ...  # type: Any
    extra_error_info = ...  # type: Any
    def __init__(self, err_msg: Optional[Any] = ..., type: str = ..., **extra_error_info) -> None: ...

class ItemDeletionException(MessageException): ...
class ObjectInvalid(Exception): ...

class ActionInputError(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any
    def __init__(self, err_msg, type: str = ...) -> None: ...

class DuplicatedSlugException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class DuplicatedIdentifierException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ObjectAttributeInvalidException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ObjectAttributeMissingException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class MalformedId(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class MalformedContents(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class UnknownContentsType(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class RequestParameterMissingException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ToolMetaParameterException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ToolMissingException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class RequestParameterInvalidException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class AuthenticationFailed(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class AuthenticationRequired(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ItemAccessibilityException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ItemOwnershipException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ConfigDoesNotAllowException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class InsufficientPermissionsException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class AdminRequiredException(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ObjectNotFound(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class DeprecatedMethod(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class Conflict(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ConfigurationError(Exception):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class InconsistentDatabase(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class InternalServerError(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class NotImplemented(MessageException):
    status_code = ...  # type: int
    err_code = ...  # type: Any

class ContainerCLIError(Exception):
    stdout = ...  # type: Any
    stderr = ...  # type: Any
    returncode = ...  # type: Any
    command = ...  # type: Any
    subprocess_command = ...  # type: Any
    def __init__(self, msg: Optional[Any] = ..., stdout: Optional[Any] = ..., stderr: Optional[Any] = ..., returncode: Optional[Any] = ..., command: Optional[Any] = ..., subprocess_command: Optional[Any] = ..., **kwargs) -> None: ...

class ContainerNotFound(Exception):
    container_id = ...  # type: Any
    def __init__(self, msg: Optional[Any] = ..., container_id: Optional[Any] = ..., **kwargs) -> None: ...

class ContainerImageNotFound(Exception):
    image = ...  # type: Any
    def __init__(self, msg: Optional[Any] = ..., image: Optional[Any] = ..., **kwargs) -> None: ...

class ContainerRunError(Exception):
    image = ...  # type: Any
    command = ...  # type: Any
    def __init__(self, msg: Optional[Any] = ..., image: Optional[Any] = ..., command: Optional[Any] = ..., **kwargs) -> None: ...



# Stubs for galaxy.tools.loader_directory (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def find_possible_tools_from_path(path, recursive: bool = ..., enable_beta_formats: bool = ...): ...
def load_tool_sources_from_path(path, load_exception_handler: Any = ..., recursive: bool = ..., register_load_errors: bool = ...): ...
def load_tool_elements_from_path(path, load_exception_handler: Any = ..., recursive: bool = ..., register_load_errors: bool = ...): ...
def is_tool_load_error(obj): ...
def looks_like_a_tool_xml(path): ...
def is_a_yaml_with_class(path, classes): ...
def looks_like_a_tool_yaml(path): ...
def looks_like_a_cwl_artifact(path, classes: Optional[Any] = ...): ...
def looks_like_a_tool_cwl(path): ...



# Stubs for galaxy.tools.deps.resolvers.tool_shed_packages (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .galaxy_packages import BaseGalaxyPackageDependencyResolver
from .resolver_mixins import UsesInstalledRepositoriesMixin

class ToolShedPackageDependencyResolver(BaseGalaxyPackageDependencyResolver, UsesInstalledRepositoriesMixin):
    resolver_type = ...  # type: str
    dependency_type = ...  # type: Any
    resolves_simple_dependencies = ...  # type: bool
    def __init__(self, dependency_manager, **kwds) -> None: ...



# Stubs for prov.identifier (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

__email__: str

class Identifier:
    def __init__(self, uri: Any) -> None: ...
    @property
    def uri(self): ...
    def __eq__(self, other: Any): ...
    def __hash__(self): ...
    def provn_representation(self): ...

class QualifiedName(Identifier):
    def __init__(self, namespace: Any, localpart: Any) -> None: ...
    @property
    def namespace(self): ...
    @property
    def localpart(self): ...
    def __hash__(self): ...
    def provn_representation(self): ...

class Namespace:
    def __init__(self, prefix: Any, uri: Any) -> None: ...
    @property
    def uri(self): ...
    @property
    def prefix(self): ...
    def contains(self, identifier: Any): ...
    def qname(self, identifier: Any): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __hash__(self): ...
    def __getitem__(self, localpart: Any): ...



"""Test sandboxjs.py and related code."""
import logging
import os
import shutil
import threading
from pathlib import Path
from typing import Any, List

import pytest

from cwltool import sandboxjs
from cwltool.factory import Factory
from cwltool.loghandler import _logger, configure_logging

from .util import get_data, needs_podman, needs_singularity

node_versions = [
    ("v0.8.26\n", False),
    ("v0.10.25\n", False),
    ("v0.10.26\n", True),
    ("v4.4.2\n", True),
    ("v7.7.3\n", True),
]

configure_logging(_logger.handlers[-1], False, True, True, True)
_logger.setLevel(logging.DEBUG)


@pytest.mark.parametrize("version,supported", node_versions)
def test_node_version(version: str, supported: bool, mocker: Any) -> None:
    mocked_subprocess = mocker.patch("cwltool.sandboxjs.subprocess")
    mocked_subprocess.check_output = mocker.Mock(return_value=version)

    assert sandboxjs.check_js_threshold_version("node") == supported


def test_value_from_two_concatenated_expressions() -> None:
    sandboxjs.have_node_slim = False
    sandboxjs.localdata = threading.local()
    factory = Factory()
    echo = factory.make(get_data("tests/wf/vf-concat.cwl"))
    file = {"class": "File", "location": get_data("tests/wf/whale.txt")}

    assert echo(file1=file) == {"out": "a string\n"}


def hide_nodejs(temp_dir: Path) -> str:
    """Generate a new PATH that hides node{js,}."""
    paths: List[str] = os.environ.get("PATH", "").split(":")
    names: List[str] = []
    for name in ("nodejs", "node"):
        path = shutil.which(name)
        if path:
            names.append(path)
    for name in names:
        dirname = os.path.dirname(name)
        if dirname in paths:
            paths.remove(dirname)
            new_dir = temp_dir / os.path.basename(dirname)
            new_dir.mkdir()
            for entry in os.listdir(dirname):
                if entry not in ("nodejs", "node"):
                    os.symlink(os.path.join(dirname, entry), new_dir / entry)
            paths.append(str(new_dir))
    return ":".join(paths)


@needs_podman
def test_value_from_two_concatenated_expressions_podman(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    """Javascript test using podman."""
    sandboxjs.have_node_slim = False
    sandboxjs.localdata = threading.local()
    new_paths = hide_nodejs(tmp_path)
    factory = Factory()
    factory.loading_context.podman = True
    factory.loading_context.debug = True
    factory.runtime_context.debug = True
    with monkeypatch.context() as m:
        m.setenv("PATH", new_paths)
        echo = factory.make(get_data("tests/wf/vf-concat.cwl"))
        file = {"class": "File", "location": get_data("tests/wf/whale.txt")}
        assert echo(file1=file) == {"out": "a string\n"}


@needs_singularity
def test_value_from_two_concatenated_expressions_singularity(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    """Javascript test using Singularity."""
    sandboxjs.have_node_slim = False
    sandboxjs.localdata = threading.local()
    new_paths = hide_nodejs(tmp_path)
    factory = Factory()
    factory.loading_context.singularity = True
    factory.loading_context.debug = True
    factory.runtime_context.debug = True
    with monkeypatch.context() as m:
        m.setenv("PATH", new_paths)
        echo = factory.make(get_data("tests/wf/vf-concat.cwl"))
        file = {"class": "File", "location": get_data("tests/wf/whale.txt")}
        assert echo(file1=file) == {"out": "a string\n"}


def test_caches_js_processes(mocker: Any) -> None:
    sandboxjs.exec_js_process("7", context="{}")

    mocked_new_js_proc = mocker.patch("cwltool.sandboxjs.new_js_proc")
    sandboxjs.exec_js_process("7", context="{}")

    mocked_new_js_proc.assert_not_called()



# Stubs for galaxy.util.json (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def json_fix(val): ...
def safe_dumps(*args, **kwargs): ...
def validate_jsonrpc_request(request, regular_methods, notification_methods): ...
def validate_jsonrpc_response(response, id: Optional[Any] = ...): ...
def jsonrpc_request(method, params: Optional[Any] = ..., id: Optional[Any] = ..., jsonrpc: str = ...): ...
def jsonrpc_response(request: Optional[Any] = ..., id: Optional[Any] = ..., result: Optional[Any] = ..., error: Optional[Any] = ..., jsonrpc: str = ...): ...



# Stubs for prov.serializers.provjson (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov.constants import *
import json
from prov.serializers import Error, Serializer
from typing import Any

logger: Any
__email__: str

class ProvJSONException(Error): ...

class AnonymousIDGenerator:
    def __init__(self) -> None: ...
    def get_anon_id(self, obj: Any, local_prefix: str = ...): ...

LITERAL_XSDTYPE_MAP: Any

class ProvJSONSerializer(Serializer):
    def serialize(self, stream: Any, **kwargs: Any) -> None: ...
    def deserialize(self, stream: Any, **kwargs: Any): ...

class ProvJSONEncoder(json.JSONEncoder):
    def default(self, o: Any): ...

class ProvJSONDecoder(json.JSONDecoder):
    def decode(self, s: Any, *args: Any, **kwargs: Any): ...

def valid_qualified_name(bundle: Any, value: Any): ...
def encode_json_document(document: Any): ...
def encode_json_container(bundle: Any): ...
def decode_json_document(content: Any, document: Any) -> None: ...
def decode_json_container(jc: Any, bundle: Any) -> None: ...
def encode_json_representation(value: Any): ...
def decode_json_representation(literal: Any, bundle: Any): ...
def literal_json_representation(literal: Any): ...



# Stubs for networkx.generators.directed (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def gn_graph(n, kernel: Optional[Any] = ..., create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def gnr_graph(n, p, create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def gnc_graph(n, create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def scale_free_graph(n, alpha: float = ..., beta: float = ..., gamma: float = ..., delta_in: float = ..., delta_out: int = ..., create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def random_k_out_graph(n, k, alpha, self_loops: bool = ..., seed: Optional[Any] = ...): ...



# Stubs for galaxy.jobs.metrics.collectl.cli (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class CollectlCli:
    mode = ...  # type: Any
    command_args = ...  # type: Any
    def __init__(self, **kwargs) -> None: ...
    def build_command_line(self): ...
    def run(self, stdout: Any = ..., stderr: Any = ...): ...



"""Tests for --make-template."""

from schema_salad.sourceline import cmap

from cwltool import main


def test_anonymous_record() -> None:
    inputs = cmap([{"type": "record", "fields": []}])
    assert main.generate_example_input(inputs, None) == ({}, "Anonymous record type.")



from .generate import arcp_hash as arcp_hash, arcp_location as arcp_location, arcp_name as arcp_name, arcp_random as arcp_random, arcp_uuid as arcp_uuid
from .parse import is_arcp_uri as is_arcp_uri, parse_arcp as parse_arcp

ARCP: str
NI: str
NIH: str



# Stubs for networkx.classes.digraph (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.classes.graph import Graph
from typing import Any, Optional

class DiGraph(Graph):
    node_dict_factory: Any = ...
    adjlist_outer_dict_factory: Any = ...
    adjlist_inner_dict_factory: Any = ...
    edge_attr_dict_factory: Any = ...
    root_graph: Any = ...
    graph: Any = ...
    def __init__(self, incoming_graph_data: Optional[Any] = ..., **attr) -> None: ...
    @property
    def adj(self): ...
    @property
    def succ(self): ...
    @property
    def pred(self): ...
    def add_node(self, node_for_adding, **attr): ...
    def add_nodes_from(self, nodes_for_adding, **attr): ...
    def remove_node(self, n): ...
    def remove_nodes_from(self, nodes): ...
    def add_edge(self, u_of_edge, v_of_edge, **attr): ...
    def add_edges_from(self, ebunch_to_add, **attr): ...
    def remove_edge(self, u, v): ...
    def remove_edges_from(self, ebunch): ...
    def has_successor(self, u, v): ...
    def has_predecessor(self, u, v): ...
    def successors(self, n): ...
    neighbors: Any = ...
    def predecessors(self, n): ...
    @property
    def edges(self): ...
    out_edges: Any = ...
    @property
    def in_edges(self): ...
    @property
    def degree(self): ...
    @property
    def in_degree(self): ...
    @property
    def out_degree(self): ...
    def clear(self): ...
    def is_multigraph(self): ...
    def is_directed(self): ...
    def fresh_copy(self): ...
    def copy(self, as_view: bool = ...): ...
    def to_undirected(self, reciprocal: bool = ..., as_view: bool = ...): ...
    def subgraph(self, nodes): ...
    def reverse(self, copy: bool = ...): ...



# Stubs for galaxy.objectstore.pulsar (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from ..objectstore import ObjectStore as ObjectStore

ObjectStoreClientManager = ...  # type: Any

class PulsarObjectStore(ObjectStore):
    pulsar_client = ...  # type: Any
    def __init__(self, config, config_xml) -> None: ...
    def exists(self, obj, **kwds): ...
    def file_ready(self, obj, **kwds): ...
    def create(self, obj, **kwds): ...
    def empty(self, obj, **kwds): ...
    def size(self, obj, **kwds): ...
    def delete(self, obj, **kwds): ...
    def get_data(self, obj, **kwds): ...
    def get_filename(self, obj, **kwds): ...
    def update_from_file(self, obj, **kwds): ...
    def get_store_usage_percent(self): ...
    def get_object_url(self, obj, extra_dir: Optional[Any] = ..., extra_dir_at_root: bool = ..., alt_name: Optional[Any] = ...): ...
    def shutdown(self): ...



# Stubs for galaxy.util.specs (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def to_str_or_none(value): ...
def to_bool_or_none(value): ...
def to_bool(value): ...
def to_float_or_none(value): ...
def is_in(*args): ...



# Stubs for networkx.algorithms.community.community_generators (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def LFR_benchmark_graph(n, tau1, tau2, mu, average_degree: Optional[Any] = ..., min_degree: Optional[Any] = ..., max_degree: Optional[Any] = ..., min_community: Optional[Any] = ..., max_community: Optional[Any] = ..., tol: float = ..., max_iters: int = ..., seed: Optional[Any] = ...): ...



from typing import Any, List

PY3: Any
str_type = str
GRAPH_ATTRIBUTES: Any
EDGE_ATTRIBUTES: Any
NODE_ATTRIBUTES: Any
CLUSTER_ATTRIBUTES: Any
DEFAULT_PROGRAMS: Any

def is_windows() -> bool: ...
def is_anaconda() -> bool: ...
def get_executable_extension() -> str: ...
def call_graphviz(program, arguments, working_dir, **kwargs): ...

class frozendict(dict):
    __delitem__: Any
    __setitem__: Any
    clear: Any
    pop: Any
    popitem: Any
    setdefault: Any
    update: Any
    def __new__(cls, *args, **kw): ...
    def __init__(self, *args, **kw) -> None: ...
    def __hash__(self): ...

dot_keywords: Any
id_re_alpha_nums: Any
id_re_alpha_nums_with_ports: Any
id_re_num: Any
id_re_with_port: Any
id_re_dbl_quoted: Any
id_re_html: Any

def needs_quotes(s): ...
def quote_if_necessary(s): ...
def graph_from_dot_data(s) -> List['Dot']: ...
def graph_from_dot_file(path, encoding: Any | None = ...): ...
def graph_from_edges(edge_list, node_prefix: str = ..., directed: bool = ...): ...
def graph_from_adjacency_matrix(matrix, node_prefix: str = ..., directed: bool = ...): ...
def graph_from_incidence_matrix(matrix, node_prefix: str = ..., directed: bool = ...): ...

class Common:
    def __get_attribute__(self, attr): ...
    def set_parent_graph(self, parent_graph) -> None: ...
    def get_parent_graph(self): ...
    def set(self, name, value) -> None: ...
    def get(self, name): ...
    def get_attributes(self): ...
    def set_sequence(self, seq) -> None: ...
    def get_sequence(self): ...
    def create_attribute_methods(self, obj_attributes: List[str]): ...

class Error(Exception):
    value: Any
    def __init__(self, value) -> None: ...

class InvocationException(Exception):
    value: Any
    def __init__(self, value) -> None: ...

class Node(Common):
    obj_dict: Any
    def __init__(self, name: str = ..., obj_dict: Any | None = ..., **attrs) -> None: ...
    def set_name(self, node_name) -> None: ...
    def get_name(self): ...
    def get_port(self): ...
    def add_style(self, style) -> None: ...
    def to_string(self) -> str: ...

class Edge(Common):
    obj_dict: Any
    def __init__(self, src: str = ..., dst: str = ..., obj_dict: Any | None = ..., **attrs) -> None: ...
    def get_source(self): ...
    def get_destination(self): ...
    def __hash__(self): ...
    def __eq__(self, edge): ...
    def parse_node_ref(self, node_str): ...
    def to_string(self) -> str: ...

class Graph(Common):
    obj_dict: Any
    def __init__(self, graph_name: str = ..., obj_dict: Any | None = ..., graph_type: str = ..., strict: bool = ..., suppress_disconnected: bool = ..., simplify: bool = ..., **attrs) -> None: ...
    def get_graph_type(self): ...
    def get_top_graph_type(self): ...
    def set_graph_defaults(self, **attrs) -> None: ...
    def get_graph_defaults(self, **attrs): ...
    def set_node_defaults(self, **attrs) -> None: ...
    def get_node_defaults(self, **attrs): ...
    def set_edge_defaults(self, **attrs) -> None: ...
    def get_edge_defaults(self, **attrs): ...
    def set_simplify(self, simplify) -> None: ...
    def get_simplify(self): ...
    def set_type(self, graph_type) -> None: ...
    def get_type(self): ...
    def set_name(self, graph_name) -> None: ...
    def get_name(self): ...
    def set_strict(self, val) -> None: ...
    def get_strict(self, val): ...
    def set_suppress_disconnected(self, val) -> None: ...
    def get_suppress_disconnected(self, val): ...
    def get_next_sequence_number(self): ...
    def add_node(self, graph_node) -> None: ...
    def del_node(self, name, index: Any | None = ...): ...
    def get_node(self, name): ...
    def get_nodes(self): ...
    def get_node_list(self): ...
    def add_edge(self, graph_edge) -> None: ...
    def del_edge(self, src_or_list, dst: Any | None = ..., index: Any | None = ...): ...
    def get_edge(self, src_or_list, dst: Any | None = ...): ...
    def get_edges(self): ...
    def get_edge_list(self): ...
    def add_subgraph(self, sgraph) -> None: ...
    def get_subgraph(self, name): ...
    def get_subgraphs(self): ...
    def get_subgraph_list(self): ...
    def set_parent_graph(self, parent_graph) -> None: ...
    def to_string(self) -> str: ...

class Subgraph(Graph):
    def __init__(self, graph_name: str = ..., obj_dict: Any | None = ..., suppress_disconnected: bool = ..., simplify: bool = ..., **attrs) -> None: ...

class Cluster(Graph):
    def __init__(self, graph_name: str = ..., obj_dict: Any | None = ..., suppress_disconnected: bool = ..., simplify: bool = ..., **attrs) -> None: ...

class Dot(Graph):
    shape_files: Any
    formats: Any
    prog: str
    def __init__(self, *argsl, **argsd): ...
    def set_shape_files(self, file_paths) -> None: ...
    def set_prog(self, prog) -> None: ...
    def write(self, path, prog: Any | None = ..., format: str = ..., encoding: Any | None = ...): ...
    def create(self, prog: Any | None = ..., format: str = ..., encoding: Any | None = ...): ...



# Stubs for prov.serializers.provxml (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from prov.constants import *
import prov.identifier
from typing import Any, Optional

__email__: str
logger: Any
FULL_NAMES_MAP: Any
FULL_PROV_RECORD_IDS_MAP: Any
XML_XSD_URI: str

class ProvXMLException(prov.Error): ...

class ProvXMLSerializer(prov.serializers.Serializer):
    def serialize(self, stream: Any, force_types: bool = ..., **kwargs: Any) -> None: ...
    def serialize_bundle(self, bundle: Any, element: Optional[Any] = ..., force_types: bool = ...): ...
    def deserialize(self, stream: Any, **kwargs: Any): ...
    def deserialize_subtree(self, xml_doc: Any, bundle: Any): ...

def xml_qname_to_QualifiedName(element: Any, qname_str: Any): ...



# Stubs for networkx.algorithms.centrality.betweenness (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def betweenness_centrality(G, k: Optional[Any] = ..., normalized: bool = ..., weight: Optional[Any] = ..., endpoints: bool = ..., seed: Optional[Any] = ...): ...
def edge_betweenness_centrality(G, k: Optional[Any] = ..., normalized: bool = ..., weight: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def edge_betweenness(G, k: Optional[Any] = ..., normalized: bool = ..., weight: Optional[Any] = ..., seed: Optional[Any] = ...): ...



# Stubs for galaxy.jobs.metrics.formatting (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class JobMetricFormatter:
    def format(self, key, value): ...

def seconds_to_str(value): ...



# Stubs for galaxy.util.heartbeat (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import threading

def get_current_thread_object_dict(): ...

class Heartbeat(threading.Thread):
    config = ...  # type: Any
    should_stop = ...  # type: bool
    period = ...  # type: Any
    fname = ...  # type: Any
    file = ...  # type: Any
    fname_nonsleeping = ...  # type: Any
    file_nonsleeping = ...  # type: Any
    pid = ...  # type: Any
    nonsleeping_heartbeats = ...  # type: Any
    wait_event = ...  # type: Any
    def __init__(self, config, name: str = ..., period: int = ..., fname: str = ...) -> None: ...
    def run(self): ...
    def open_logs(self): ...
    def close_logs(self): ...
    def dump(self): ...
    def shutdown(self): ...
    def thread_is_sleeping(self, last_stack_frame): ...
    def get_interesting_stack_frame(self, stack_frames): ...
    def print_nonsleeping(self, threads_object_dict): ...
    def dump_signal_handler(self, signum, frame): ...



# Stubs for galaxy.tools.parser.output_actions (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

log = ...  # type: Any
COLLECTION_ATTRIBUTES = ...  # type: Any

class ToolOutputActionGroup:
    tag = ...  # type: str
    parent = ...  # type: Any
    actions = ...  # type: Any
    def __init__(self, parent, config_elem) -> None: ...
    def apply_action(self, output_dataset, other_values): ...
    @property
    def tool(self): ...
    def __len__(self): ...

class ToolOutputActionConditionalWhen(ToolOutputActionGroup):
    tag = ...  # type: str
    @classmethod
    def from_elem(cls, parent, when_elem): ...
    value = ...  # type: Any
    def __init__(self, parent, config_elem, value) -> None: ...
    def is_case(self, output_dataset, other_values): ...
    def get_ref(self, output_dataset, other_values): ...
    def apply_action(self, output_dataset, other_values): ...

class ValueToolOutputActionConditionalWhen(ToolOutputActionConditionalWhen):
    tag = ...  # type: str
    def is_case(self, output_dataset, other_values): ...

class DatatypeIsInstanceToolOutputActionConditionalWhen(ToolOutputActionConditionalWhen):
    tag = ...  # type: str
    value = ...  # type: Any
    def __init__(self, parent, config_elem, value) -> None: ...
    def is_case(self, output_dataset, other_values): ...

class ToolOutputActionConditional:
    tag = ...  # type: str
    parent = ...  # type: Any
    name = ...  # type: Any
    cases = ...  # type: Any
    def __init__(self, parent, config_elem) -> None: ...
    def apply_action(self, output_dataset, other_values): ...
    @property
    def tool(self): ...

class ToolOutputAction:
    tag = ...  # type: str
    @classmethod
    def from_elem(cls, parent, elem): ...
    parent = ...  # type: Any
    default = ...  # type: Any
    option = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def apply_action(self, output_dataset, other_values): ...
    @property
    def tool(self): ...

class ToolOutputActionOption:
    tag = ...  # type: str
    @classmethod
    def from_elem(cls, parent, elem): ...
    parent = ...  # type: Any
    filters = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def get_value(self, other_values): ...
    @property
    def tool(self): ...

class NullToolOutputActionOption(ToolOutputActionOption):
    tag = ...  # type: str
    def get_value(self, other_values): ...

class FromFileToolOutputActionOption(ToolOutputActionOption):
    tag = ...  # type: str
    name = ...  # type: Any
    column = ...  # type: Any
    offset = ...  # type: Any
    separator = ...  # type: Any
    options = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def get_value(self, other_values): ...

class FromParamToolOutputActionOption(ToolOutputActionOption):
    tag = ...  # type: str
    name = ...  # type: Any
    column = ...  # type: Any
    offset = ...  # type: Any
    param_attribute = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def get_value(self, other_values): ...

class FromDataTableOutputActionOption(ToolOutputActionOption):
    tag = ...  # type: str
    name = ...  # type: Any
    missing_tool_data_table_name = ...  # type: Any
    options = ...  # type: Any
    column = ...  # type: Any
    offset = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def get_value(self, other_values): ...

class MetadataToolOutputAction(ToolOutputAction):
    tag = ...  # type: str
    name = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def apply_action(self, output_dataset, other_values): ...

class FormatToolOutputAction(ToolOutputAction):
    tag = ...  # type: str
    default = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def apply_action(self, output_dataset, other_values): ...

class ToolOutputActionOptionFilter:
    tag = ...  # type: str
    @classmethod
    def from_elem(cls, parent, elem): ...
    parent = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...
    @property
    def tool(self): ...

class ParamValueToolOutputActionOptionFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    ref = ...  # type: Any
    value = ...  # type: Any
    column = ...  # type: Any
    keep = ...  # type: Any
    compare = ...  # type: Any
    cast = ...  # type: Any
    param_attribute = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class InsertColumnToolOutputActionOptionFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    ref = ...  # type: Any
    value = ...  # type: Any
    column = ...  # type: Any
    iterate = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class MultipleSplitterFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    column = ...  # type: Any
    separator = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class ColumnStripFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    column = ...  # type: Any
    strip = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class ColumnReplaceFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    old_column = ...  # type: Any
    old_value = ...  # type: Any
    new_value = ...  # type: Any
    new_column = ...  # type: Any
    column = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class MetadataValueFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    ref = ...  # type: Any
    name = ...  # type: Any
    column = ...  # type: Any
    keep = ...  # type: Any
    compare = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class BooleanFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    column = ...  # type: Any
    keep = ...  # type: Any
    cast = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

class StringFunctionFilter(ToolOutputActionOptionFilter):
    tag = ...  # type: str
    column = ...  # type: Any
    function = ...  # type: Any
    def __init__(self, parent, elem) -> None: ...
    def filter_options(self, options, other_values): ...

action_types = ...  # type: Any
option_types = ...  # type: Any
filter_types = ...  # type: Any

def parse_cast_attribute(cast): ...
def parse_compare_type(compare): ...
def compare_eq(value1, value2): ...
def compare_neq(value1, value2): ...
def compare_gt(value1, value2): ...
def compare_gte(value1, value2): ...
def compare_lt(value1, value2): ...
def compare_lte(value1, value2): ...
def compare_in(value1, value2): ...
def compare_startswith(value1, value2): ...
def compare_endswith(value1, value2): ...
def compare_re_search(value1, value2): ...

compare_types = ...  # type: Any



# Stubs for galaxy.util.sockets (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def unused_port(range: Optional[Any] = ...): ...



from base64 import urlsafe_b64decode as urlsafe_b64decode
from typing import Any, Optional

SCHEME: str

def arcp_uuid(uuid: Any, path: str = ..., query: Optional[Any] = ..., fragment: Optional[Any] = ...): ...
def arcp_random(path: str = ..., query: Optional[Any] = ..., fragment: Optional[Any] = ..., uuid: Optional[Any] = ...): ...
def arcp_location(location: Any, path: str = ..., query: Optional[Any] = ..., fragment: Optional[Any] = ..., namespace: Any = ...): ...
def arcp_name(name: Any, path: str = ..., query: Optional[Any] = ..., fragment: Optional[Any] = ...): ...
def arcp_hash(bytes: bytes = ..., path: str = ..., query: Optional[Any] = ..., fragment: Optional[Any] = ..., hash: Optional[Any] = ...): ...



# Stubs for networkx.algorithms.centrality.current_flow_betweenness_subset (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.centrality.flow_matrix import *
from typing import Any, Optional

def current_flow_betweenness_centrality_subset(G, sources, targets, normalized: bool = ..., weight: Optional[Any] = ..., dtype: Any = ..., solver: str = ...): ...
def edge_current_flow_betweenness_centrality_subset(G, sources, targets, normalized: bool = ..., weight: Optional[Any] = ..., dtype: Any = ..., solver: str = ...): ...



# Stubs for galaxy.tools.deps.resolvers.resolver_mixins (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ..brew_exts import build_env_statements as build_env_statements, DEFAULT_HOMEBREW_ROOT as DEFAULT_HOMEBREW_ROOT, recipe_cellar_path as recipe_cellar_path
from ..resolvers import Dependency as Dependency, NullDependency as NullDependency

class UsesHomebrewMixin: ...
class UsesToolDependencyDirMixin: ...
class UsesInstalledRepositoriesMixin: ...

class HomebrewDependency(Dependency):
    commands = ...  # type: Any
    def __init__(self, commands, exact: bool = ...) -> None: ...
    @property
    def exact(self): ...
    def shell_commands(self, requirement): ...



# Stubs for networkx.algorithms.assortativity.connectivity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def average_degree_connectivity(G, source: str = ..., target: str = ..., nodes: Optional[Any] = ..., weight: Optional[Any] = ...): ...
k_nearest_neighbors = average_degree_connectivity



# Stubs for galaxy.tools.linters.tests (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from ..lint_util import is_datasource as is_datasource

def lint_tsts(tool_xml, lint_ctx): ...



# Stubs for networkx.algorithms.centrality.dispersion (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def dispersion(G, u: Optional[Any] = ..., v: Optional[Any] = ..., normalized: bool = ..., alpha: float = ..., b: float = ..., c: float = ...): ...



# Stubs for graphviz.dot (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from . import files
from typing import Any, Optional

class Dot(files.File):
    name: Any = ...
    comment: Any = ...
    graph_attr: Any = ...
    node_attr: Any = ...
    edge_attr: Any = ...
    body: Any = ...
    strict: Any = ...
    def __init__(self, name: Optional[Any] = ..., comment: Optional[Any] = ..., filename: Optional[Any] = ..., directory: Optional[Any] = ..., format: Optional[Any] = ..., engine: Optional[Any] = ..., encoding: Any = ..., graph_attr: Optional[Any] = ..., node_attr: Optional[Any] = ..., edge_attr: Optional[Any] = ..., body: Optional[Any] = ..., strict: bool = ...) -> None: ...
    def clear(self, keep_attrs: bool = ...): ...
    def __iter__(self, subgraph: bool = ...): ...
    source: Any = ...
    def node(self, name, label: Optional[Any] = ..., _attributes: Optional[Any] = ..., **attrs): ...
    def edge(self, tail_name, head_name, label: Optional[Any] = ..., _attributes: Optional[Any] = ..., **attrs): ...
    def edges(self, tail_head_iter): ...
    def attr(self, kw: Optional[Any] = ..., _attributes: Optional[Any] = ..., **attrs): ...
    def subgraph(self, graph: Optional[Any] = ..., name: Optional[Any] = ..., comment: Optional[Any] = ..., graph_attr: Optional[Any] = ..., node_attr: Optional[Any] = ..., edge_attr: Optional[Any] = ..., body: Optional[Any] = ...): ...

class SubgraphContext:
    parent: Any = ...
    graph: Any = ...
    def __init__(self, parent, kwargs) -> None: ...
    def __enter__(self): ...
    def __exit__(self, type_, value, traceback): ...

class Graph(Dot):
    @property
    def directed(self): ...

class Digraph(Dot):
    @property
    def directed(self): ...



# Stubs for galaxy.tools.deps.mulled.mulled_build_files (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from collections import namedtuple

def main(argv: Optional[Any] = ...): ...

_Line = namedtuple('_Line', ['targets', 'image_build', 'name_override'])



# Stubs for galaxy.tools.verify (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from .asserts import verify_assertions as verify_assertions
from .test_data import TestDataResolver as TestDataResolver

log = ...  # type: Any
DEFAULT_TEST_DATA_RESOLVER = ...  # type: Any

def verify(item_label, output_content, attributes, filename: Optional[Any] = ..., get_filename: Optional[Any] = ..., keep_outputs_dir: Optional[Any] = ..., verify_extra_files: Optional[Any] = ...): ...
def make_temp_fname(fname: Optional[Any] = ...): ...
def check_command(command, description): ...
def files_diff(file1, file2, attributes: Optional[Any] = ...): ...
def files_re_match(file1, file2, attributes: Optional[Any] = ...): ...
def files_re_match_multiline(file1, file2, attributes: Optional[Any] = ...): ...
def files_contains(file1, file2, attributes: Optional[Any] = ...): ...



# Stubs for galaxy.tools.deps.brew_util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ..deps import brew_exts as brew_exts

DEFAULT_TAP = ...  # type: str

class HomebrewRecipe:
    recipe = ...  # type: Any
    version = ...  # type: Any
    tap = ...  # type: Any
    def __init__(self, recipe, version, tap) -> None: ...

def requirements_to_recipes(requirements): ...
def requirement_to_recipe(requirement): ...
def requirements_to_recipe_contexts(requirements, brew_context): ...



# Stubs for networkx.generators.geometric (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def random_geometric_graph(n, radius, dim: int = ..., pos: Optional[Any] = ..., p: int = ...): ...
def soft_random_geometric_graph(n, radius, dim: int = ..., pos: Optional[Any] = ..., p: int = ..., p_dist: Optional[Any] = ...): ...
def geographical_threshold_graph(n, theta, dim: int = ..., pos: Optional[Any] = ..., weight: Optional[Any] = ..., metric: Optional[Any] = ..., p_dist: Optional[Any] = ...): ...
def waxman_graph(n, beta: float = ..., alpha: float = ..., L: Optional[Any] = ..., domain: Any = ..., metric: Optional[Any] = ...): ...
def navigable_small_world_graph(n, p: int = ..., q: int = ..., r: int = ..., dim: int = ..., seed: Optional[Any] = ...): ...
def thresholded_random_geometric_graph(n, radius, theta, dim: int = ..., pos: Optional[Any] = ..., weight: Optional[Any] = ..., p: int = ...): ...



# Stubs for galaxy.tools.deps.installable (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

log = ...  # type: Any

class InstallableContext:
    def is_installed(self): ...
    def can_install(self): ...
    def installable_description(self): ...
    def parent_path(self): ...

def ensure_installed(installable_context, install_func, auto_init): ...



# Stubs for networkx.algorithms.bipartite.edgelist (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def write_edgelist(G, path, comments: str = ..., delimiter: str = ..., data: bool = ..., encoding: str = ...): ...
def generate_edgelist(G, delimiter: str = ..., data: bool = ...): ...
def parse_edgelist(lines, comments: str = ..., delimiter: Optional[Any] = ..., create_using: Optional[Any] = ..., nodetype: Optional[Any] = ..., data: bool = ...): ...
def read_edgelist(path, comments: str = ..., delimiter: Optional[Any] = ..., create_using: Optional[Any] = ..., nodetype: Optional[Any] = ..., data: bool = ..., edgetype: Optional[Any] = ..., encoding: str = ...): ...



# Stubs for galaxy.util.compression_utils (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .checkers import is_bz2 as is_bz2, is_gzip as is_gzip

def get_fileobj(filename, mode: str = ..., gzip_only: bool = ..., bz2_only: bool = ..., zip_only: bool = ...): ...



# Stubs for galaxy.tools.linters.general (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

ERROR_VERSION_MSG = ...  # type: str
VALID_VERSION_MSG = ...  # type: str
ERROR_NAME_MSG = ...  # type: str
VALID_NAME_MSG = ...  # type: str
ERROR_ID_MSG = ...  # type: str
VALID_ID_MSG = ...  # type: str
PROFILE_PATTERN = ...  # type: Any
PROFILE_INFO_DEFAULT_MSG = ...  # type: str
PROFILE_INFO_SPECIFIED_MSG = ...  # type: str
PROFILE_INVALID_MSG = ...  # type: str
lint_tool_types = ...  # type: Any

def lint_general(tool_source, lint_ctx): ...



# Stubs for galaxy.util.xml_macros (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

REQUIRED_PARAMETER = ...  # type: Any

def load(path): ...
def template_macro_params(root): ...
def raw_tool_xml_tree(path): ...
def imported_macro_paths(root): ...

class XmlMacroDef:
    elements = ...  # type: Any
    parameters = ...  # type: Any
    def __init__(self, el) -> None: ...
    def macro_tokens(self, expand_el): ...



# Stubs for galaxy.tools.verify.test_data (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

UPDATE_TEMPLATE = ...  # type: Any
UPDATE_FAILED_TEMPLATE = ...  # type: Any
LIST_SEP = ...  # type: Any

class TestDataResolver:
    resolvers = ...  # type: Any
    def __init__(self, env_var: str = ..., environ: Any = ...) -> None: ...
    def get_filename(self, name): ...

def build_resolver(uri, environ): ...

class FileDataResolver:
    file_dir = ...  # type: Any
    def __init__(self, file_dir) -> None: ...
    def exists(self, filename): ...
    def path(self, filename): ...

class GitDataResolver(FileDataResolver):
    repository = ...  # type: Any
    updated = ...  # type: bool
    fetch_data = ...  # type: Any
    def __init__(self, repository, environ) -> None: ...
    def exists(self, filename): ...
    def update_repository(self): ...
    def execute(self, cmd): ...



# Stubs for galaxy.util (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
import collections
from .inflection import English as English, Inflector as Inflector

grp = ...  # type: Any
docutils_core = ...  # type: Any
docutils_html4css1 = ...  # type: Any
inflector = ...  # type: Any
log = ...  # type: Any
CHUNK_SIZE = ...  # type: int
DATABASE_MAX_STRING_SIZE = ...  # type: int
DATABASE_MAX_STRING_SIZE_PRETTY = ...  # type: str
gzip_magic = ...  # type: str
bz2_magic = ...  # type: str
DEFAULT_ENCODING = ...  # type: Any
NULL_CHAR = ...  # type: str
BINARY_CHARS = ...  # type: Any
FILENAME_VALID_CHARS = ...  # type: str

def remove_protocol_from_url(url): ...
def is_binary(value, binary_chars: Optional[Any] = ...): ...
def is_uuid(value): ...
def directory_hash_id(id): ...
def get_charset_from_http_headers(headers, default: Optional[Any] = ...): ...
def synchronized(func): ...
def file_iter(fname, sep: Optional[Any] = ...): ...
def file_reader(fp, chunk_size: Any = ...): ...
def unique_id(KEY_SIZE: int = ...): ...
def parse_xml(fname): ...
def parse_xml_string(xml_string): ...
def xml_to_string(elem, pretty: bool = ...): ...
def xml_element_compare(elem1, elem2): ...
def xml_element_list_compare(elem_list1, elem_list2): ...
def xml_element_to_dict(elem): ...
def pretty_print_xml(elem, level: int = ...): ...
def get_file_size(value, default: Optional[Any] = ...): ...
def shrink_stream_by_size(value, size, join_by: str = ..., left_larger: bool = ..., beginning_on_size_error: bool = ..., end_on_size_error: bool = ...): ...
def shrink_string_by_size(value, size, join_by: str = ..., left_larger: bool = ..., beginning_on_size_error: bool = ..., end_on_size_error: bool = ...): ...
def pretty_print_time_interval(time: bool = ..., precise: bool = ...): ...
def pretty_print_json(json_data, is_json_string: bool = ...): ...

valid_chars = ...  # type: Any
mapped_chars = ...  # type: Any

def restore_text(text, character_map: Any = ...): ...
def sanitize_text(text, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...
def sanitize_lists_to_string(values, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...
def sanitize_param(value, valid_characters: Any = ..., character_map: Any = ..., invalid_character: str = ...): ...

valid_filename_chars = ...  # type: Any
invalid_filenames = ...  # type: Any

def sanitize_for_filename(text, default: Optional[Any] = ...): ...
def mask_password_from_url(url): ...
def ready_name_for_url(raw_name): ...
def which(file): ...
def safe_makedirs(path): ...
def in_directory(file, directory, local_path_module: Any = ...): ...
def merge_sorted_iterables(operator, *iterables): ...

class Params:
    NEVER_SANITIZE = ...  # type: Any
    def __init__(self, params, sanitize: bool = ...) -> None: ...
    def flatten(self): ...
    def __getattr__(self, name): ...
    def get(self, key, default): ...
    def __len__(self): ...
    def __iter__(self): ...
    def update(self, values): ...

def rst_to_html(s): ...
def xml_text(root, name: Optional[Any] = ...): ...

truthy = ...  # type: Any
falsy = ...  # type: Any

def asbool(obj): ...
def string_as_bool(string): ...
def string_as_bool_or_none(string): ...
def listify(item, do_strip: bool = ...): ...
def commaify(amount): ...
def roundify(amount, sfs: int = ...): ...
def unicodify(value, encoding: Any = ..., error: str = ..., default: Optional[Any] = ...): ...
def smart_str(s, encoding: Any = ..., strings_only: bool = ..., errors: str = ...): ...
def object_to_string(obj): ...
def string_to_object(s): ...

class ParamsWithSpecs(collections.defaultdict):
    specs = ...  # type: Any
    params = ...  # type: Any
    def __init__(self, specs: Optional[Any] = ..., params: Optional[Any] = ...) -> None: ...
    def __missing__(self, name): ...
    def __getattr__(self, name): ...

def compare_urls(url1, url2, compare_scheme: bool = ..., compare_hostname: bool = ..., compare_path: bool = ...): ...
def read_dbnames(filename): ...
def read_build_sites(filename, check_builds: bool = ...): ...
def relativize_symlinks(path, start: Optional[Any] = ..., followlinks: bool = ...): ...
def stringify_dictionary_keys(in_dict): ...
def recursively_stringify_dictionary_keys(d): ...
def mkstemp_ln(src, prefix: str = ...): ...
def umask_fix_perms(path, umask, unmasked_perms, gid: Optional[Any] = ...): ...
def docstring_trim(docstring): ...
def nice_size(size): ...
def size_to_bytes(size): ...
def send_mail(frm, to, subject, body, config, html: Optional[Any] = ...): ...
def force_symlink(source, link_name): ...
def move_merge(source, target): ...
def safe_str_cmp(a, b): ...

galaxy_root_path = ...  # type: Any

def galaxy_directory(): ...
def config_directories_from_setting(directories_setting, galaxy_root: Any = ...): ...
def parse_int(value, min_val: Optional[Any] = ..., max_val: Optional[Any] = ..., default: Optional[Any] = ..., allow_none: bool = ...): ...
def parse_non_hex_float(s): ...
def build_url(base_url, port: int = ..., scheme: str = ..., pathspec: Optional[Any] = ..., params: Optional[Any] = ..., doseq: bool = ...): ...
def url_get(base_url, password_mgr: Optional[Any] = ..., pathspec: Optional[Any] = ..., params: Optional[Any] = ...): ...
def download_to_file(url, dest_file_path, timeout: int = ..., chunk_size: Any = ...): ...
def safe_relpath(path): ...

class ExecutionTimer:
    begin = ...  # type: Any
    def __init__(self) -> None: ...
    @property
    def elapsed(self): ...



# Stubs for galaxy.tools.deps.views (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class DependencyResolversView:
    def __init__(self, app) -> None: ...
    def index(self): ...
    def show(self, index): ...
    def reload(self): ...
    def manager_requirements(self): ...
    def resolver_requirements(self, index): ...
    def manager_dependency(self, **kwds): ...
    def resolver_dependency(self, index, **kwds): ...
    def show_dependencies(self, tool_requirements_d, installed_tool_dependencies: Optional[Any] = ...): ...
    def uninstall_dependencies(self, index: Optional[Any] = ..., **payload): ...
    @property
    def unused_dependency_paths(self): ...
    def remove_unused_dependency_paths(self, envs): ...
    def install_dependencies(self, requirements): ...
    def install_dependency(self, index: Optional[Any] = ..., **payload): ...
    @property
    def installable_resolvers(self): ...
    @property
    def uninstallable_resolvers(self): ...
    @property
    def tool_ids_by_requirements(self): ...
    @property
    def toolbox_requirements_status(self): ...
    def get_requirements_status(self, tool_requirements_d, installed_tool_dependencies: Optional[Any] = ...): ...
    def clean(self, index: Optional[Any] = ..., **kwds): ...



# Stubs for networkx.algorithms.assortativity.mixing (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def attribute_mixing_dict(G, attribute, nodes: Optional[Any] = ..., normalized: bool = ...): ...
def attribute_mixing_matrix(G, attribute, nodes: Optional[Any] = ..., mapping: Optional[Any] = ..., normalized: bool = ...): ...
def degree_mixing_dict(G, x: str = ..., y: str = ..., weight: Optional[Any] = ..., nodes: Optional[Any] = ..., normalized: bool = ...): ...
def degree_mixing_matrix(G, x: str = ..., y: str = ..., weight: Optional[Any] = ..., nodes: Optional[Any] = ..., normalized: bool = ...): ...
def numeric_mixing_matrix(G, attribute, nodes: Optional[Any] = ..., normalized: bool = ...): ...
def mixing_dict(xy, normalized: bool = ...): ...



# Stubs for galaxy.tools.loader (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from galaxy.util.xml_macros import imported_macro_paths as imported_macro_paths, raw_tool_xml_tree as raw_tool_xml_tree, template_macro_params as template_macro_params

load_tool = ...  # type: Any



# Stubs for galaxy.tools.deps.requirements (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Dict, List, Optional, Text

DEFAULT_REQUIREMENT_TYPE = ...  # type: str
DEFAULT_REQUIREMENT_VERSION = ...  # type: Any

class ToolRequirement:
    name = ...  # type: Any
    type = ...  # type: Any
    version = ...  # type: Any
    specs = ...  # type: Any
    def __init__(self, name: Optional[Any] = ..., type: Optional[Any] = ..., version: Optional[Any] = ..., specs: Any = ...) -> None: ...
    def to_dict(self) -> Dict[str, Any]: ...
    def copy(self): ...
    @staticmethod
    def from_dict(dict: Dict[str, Any]) -> ToolRequirement: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self): ...

class RequirementSpecification:
    uri = ...  # type: Any
    version = ...  # type: Any
    def __init__(self, uri, version: Optional[Any] = ...) -> None: ...
    @property
    def specifies_version(self): ...
    @property
    def short_name(self): ...
    def to_dict(self): ...
    @staticmethod
    def from_dict(dict: Dict[str, Any]) -> ToolRequirements: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self): ...

class ToolRequirements:
    tool_requirements = ...  # type: Any
    def __init__(self, tool_requirements: Optional[Any] = ...) -> None: ...
    @staticmethod
    def from_list(requirements: List[ToolRequirement]) -> ToolRequirements: ...
    @property
    def resolvable(self): ...
    @property
    def packages(self): ...
    def to_list(self) -> List[ToolRequirement]: ...
    def append(self, requirement): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __iter__(self): ...
    def __getitem__(self, ii): ...
    def __len__(self): ...
    def __hash__(self): ...

class ToolRequirementsException(Exception): ...

DEFAULT_CONTAINER_TYPE = ...  # type: str
DEFAULT_CONTAINER_RESOLVE_DEPENDENCIES = ...  # type: bool
DEFAULT_CONTAINER_SHELL = ...  # type: str

class ContainerDescription:
    identifier = ...  # type: Text
    type = ...  # type: Any
    resolve_dependencies = ...  # type: Any
    shell = ...  # type: Any
    def __init__(self, identifier: Optional[Any] = ..., type: Any = ..., resolve_dependencies: Any = ..., shell: Any = ...) -> None: ...
    def to_dict(self): ...
    @staticmethod
    def from_dict(dict): ...

def parse_requirements_from_dict(root_dict): ...
def parse_requirements_from_xml(xml_root): ...
def container_from_element(container_elem): ...



# Stubs for galaxy.tools.deps.mulled.mulled_build_tool (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def main(argv: Optional[Any] = ...): ...
def requirements_to_mulled_targets(requirements): ...
